@@ -31,7 +31,8 @@ from kid import __version__ as kidversion
 from elementtree import ElementTree
 import os
 import sys
-import sre
+import re
+import locale
 
 def summarizestats(statslist, totalstats=None):
   if totalstats is None:
@@ -47,7 +48,7 @@ def shortdescription(descr):
   stopsign = descr.find("<br")
   if stopsign >= 0:
     descr = descr[:stopsign]
-  return sre.sub("<[^>]*>", "", descr).strip()
+  return re.sub("<[^>]*>", "", descr).strip()
   
 class AboutPage(pagelayout.PootlePage):
   """the bar at the side describing current login details etc"""
@@ -86,6 +87,7 @@ class PootleIndex(pagelayout.PootlePage):
     self.potree = potree
     self.localize = session.localize
     self.nlocalize = session.nlocalize
+    self.tr_lang = session.tr_lang
     templatename = "index"
     description = getattr(session.instance, "description")
     meta_description = shortdescription(description)
@@ -96,7 +98,8 @@ class PootleIndex(pagelayout.PootlePage):
     instancetitle = getattr(session.instance, "title", session.localize("Pootle Demo"))
     pagetitle = instancetitle
     sessionvars = {"status": session.status, "isopen": session.isopen, "issiteadmin": session.issiteadmin()}
-    languages = [{"code": code, "name": name, "sep": ", "} for code, name in self.potree.getlanguages()]
+    languages = [{"code": code, "name": self.tr_lang(name), "sep": ", "} for code, name in self.potree.getlanguages()]
+    languages.sort(cmp=locale.strcoll, key=lambda dict: dict["name"])
     if languages:
       languages[-1]["sep"] = ""
     templatevars = {"pagetitle": pagetitle, "description": description, 
@@ -125,6 +128,7 @@ class UserIndex(pagelayout.PootlePage):
   def __init__(self, potree, session):
     self.potree = potree
     self.session = session
+    self.tr_lang = session.tr_lang
     self.localize = session.localize
     self.nlocalize = session.nlocalize
     pagetitle = self.localize("User Page for: %s", session.username)
@@ -162,7 +166,8 @@ class UserIndex(pagelayout.PootlePage):
                             "isprojectadmin": isprojectadmin, "sep": "<br />"})
       if langlinks:
         langlinks[-1]["sep"] = ""
-      quicklinks.append({"code": languagecode, "name": languagename, "projects": langlinks})
+      quicklinks.append({"code": languagecode, "name": self.tr_lang(languagename), "projects": langlinks})
+      quicklinks.sort(cmp=locale.strcoll, key=lambda dict: dict["name"])
     return quicklinks
 
 class ProjectsIndex(PootleIndex):
@@ -184,6 +189,7 @@ class LanguageIndex(pagelayout.PootleNavPage):
     self.languagecode = languagecode
     self.localize = session.localize
     self.nlocalize = session.nlocalize
+    self.tr_lang = session.tr_lang
     self.languagename = self.potree.getlanguagename(self.languagecode)
     self.initpagestats()
     languageprojects = self.getprojects()
@@ -195,12 +201,12 @@ class LanguageIndex(pagelayout.PootleNavPage):
     # l10n: The first parameter is the name of the installation
     # l10n: The second parameter is the name of the project/language
     # l10n: This is used as a page title. Most languages won't need to change this
-    pagetitle =  self.localize("%s: %s", instancetitle, self.languagename)
+    pagetitle =  self.localize("%s: %s", instancetitle, self.tr_lang(self.languagename))
     templatename = "language"
     adminlink = self.localize("Admin")
     sessionvars = {"status": session.status, "isopen": session.isopen, "issiteadmin": session.issiteadmin()}
     templatevars = {"pagetitle": pagetitle,
-        "language": {"code": languagecode, "name": self.languagename, "stats": languagestats, "info": languageinfo},
+        "language": {"code": languagecode, "name": self.tr_lang(self.languagename), "stats": languagestats, "info": languageinfo},
         "projects": languageprojects, 
         "statsheadings": self.getstatsheadings(),
         "session": sessionvars, "instancetitle": instancetitle}
@@ -212,7 +218,7 @@ class LanguageIndex(pagelayout.PootleNavPage):
     nplurals = self.potree.getlanguagenplurals(self.languagecode)
     pluralequation = self.potree.getlanguagepluralequation(self.languagecode)
     infoparts = [(self.localize("Language Code"), self.languagecode),
-                 (self.localize("Language Name"), self.languagename),
+                 (self.localize("Language Name"), self.tr_lang(self.languagename)),
                  # (self.localize("Special Characters"), specialchars),
                  (self.localize("Number of Plurals"), str(nplurals)),
                  (self.localize("Plural Equation"), pluralequation),
@@ -246,6 +252,7 @@ class ProjectLanguageIndex(pagelayout.PootleNavPage):
     self.projectcode = projectcode
     self.localize = session.localize
     self.nlocalize = session.nlocalize
+    self.tr_lang = session.tr_lang
     self.initpagestats()
     languages = self.getlanguages()
     average = self.getpagestats()
@@ -276,6 +283,7 @@ class ProjectLanguageIndex(pagelayout.PootleNavPage):
     languages = self.potree.getlanguages(self.projectcode)
     self.languagecount = len(languages)
     languageitems = [self.getlanguageitem(languagecode, languagename) for languagecode, languagename in languages]
+    languageitems.sort(cmp=locale.strcoll, key=lambda dict: dict["title"])
     for n, item in enumerate(languageitems):
       item["parity"] = ["even", "odd"][n % 2]
     return languageitems
@@ -286,7 +294,7 @@ class ProjectLanguageIndex(pagelayout.PootleNavPage):
     quickstats = language.getquickstats()
     data = self.getstats(language, quickstats, len(language.pofilenames))
     self.updatepagestats(data["translatedwords"], data["totalwords"])
-    return {"code": languagecode, "icon": "language", "href": href, "title": languagename, "data": data}
+    return {"code": languagecode, "icon": "language", "href": href, "title": self.tr_lang(languagename), "data": data}
 
 class ProjectIndex(pagelayout.PootleNavPage):
   """The main page of a project in a specific language"""
@@ -295,6 +303,7 @@ class ProjectIndex(pagelayout.PootleNavPage):
     self.session = session
     self.localize = session.localize
     self.nlocalize = session.nlocalize
+    self.tr_lang = session.tr_lang
     self.rights = self.project.getrights(self.session)
     message = argdict.get("message", "")
     if dirfilter == "":
@@ -341,12 +350,12 @@ class ProjectIndex(pagelayout.PootleNavPage):
       childitems = self.getchilditems(dirfilter)
     instancetitle = getattr(session.instance, "title", session.localize("Pootle Demo"))
     # l10n: The first parameter is the name of the installation (like "Pootle")
-    pagetitle = self.localize("%s: Project %s, Language %s", instancetitle, self.project.projectname, self.project.languagename)
+    pagetitle = self.localize("%s: Project %s, Language %s", instancetitle, self.project.projectname, self.tr_lang(self.project.languagename))
     templatename = "fileindex"
     sessionvars = {"status": session.status, "isopen": session.isopen, "issiteadmin": session.issiteadmin()}
     templatevars = {"pagetitle": pagetitle,
         "project": {"code": self.project.projectcode, "name": self.project.projectname},
-        "language": {"code": self.project.languagecode, "name": self.project.languagename},
+        "language": {"code": self.project.languagecode, "name": self.tr_lang(self.project.languagename)},
         # optional sections, will appear if these values are replaced
         "assign": None, "goals": None, "upload": None,
         "search": {"title": self.localize("Search")}, "message": message,
@@ -394,11 +403,21 @@ class ProjectIndex(pagelayout.PootleNavPage):
       print "removed %d assigns from %s" % (assigncount, assignedto)
       del self.argdict["removeassigns"]
     if "doupload" in self.argdict:
+      extensiontypes = ("xlf", "xlff", "xliff", "po")
+      if "Yes" in self.argdict.pop("dooverwrite", []):
+        overwrite = True
+      else:
+        overwrite = False
       uploadfile = self.argdict.pop("uploadfile", None)
+      # multiple translation file extensions check
+      if filter(uploadfile.filename.endswith, extensiontypes):
+       transfiles = True
+      else:
+       transfiles = False
       if not uploadfile.filename:
         raise ValueError(self.localize("Cannot upload file, no file attached"))
-      if uploadfile.filename.endswith(".po") or uploadfile.filename.endswith(".xlf"):
-        self.project.uploadfile(self.session, self.dirname, uploadfile.filename, uploadfile.contents)
+      if transfiles:
+        self.project.uploadfile(self.session, self.dirname, uploadfile.filename, uploadfile.contents, overwrite)
       elif uploadfile.filename.endswith(".zip"):
         self.project.uploadarchive(self.session, self.dirname, uploadfile.contents)
       else:
@@ -408,19 +427,19 @@ class ProjectIndex(pagelayout.PootleNavPage):
       updatefile = self.argdict.pop("updatefile", None)
       if not updatefile:
         raise ValueError("cannot update file, no file specified")
-      if updatefile.endswith(".po"):
+      if updatefile.endswith("." + self.project.fileext):
         self.project.updatepofile(self.session, self.dirname, updatefile)
       else:
-        raise ValueError("can only update PO files")
+        raise ValueError("can only update files with extension ." + self.project.fileext)
       del self.argdict["doupdate"]
     if "docommit" in self.argdict:
       commitfile = self.argdict.pop("commitfile", None)
       if not commitfile:
         raise ValueError("cannot commit file, no file specified")
-      if commitfile.endswith(".po"):
+      if commitfile.endswith("." + self.project.fileext):
         self.project.commitpofile(self.session, self.dirname, commitfile)
       else:
-        raise ValueError("can only commit PO files")
+        raise ValueError("can only commit files with extension ." + self.project.fileext)
       del self.argdict["docommit"]
     if "doaddgoal" in self.argdict:
       goalname = self.argdict.pop("newgoal", None)
@@ -523,9 +542,23 @@ class ProjectIndex(pagelayout.PootleNavPage):
 
   def getuploadbox(self):
     """adds a box that lets the user assign strings"""
-    return {"title": self.localize("Upload File"),
+    uploadbox = {
+            "title": self.localize("Upload File"),
             "file_title": self.localize("Select file to upload"),
-            "button": self.localize("Upload File")}
+            "upload_button": self.localize("Upload File")
+            }
+    if "admin" in self.rights or "overwrite" in self.rights:
+      uploadbox.update({
+            #l10n: radio button text
+            "overwrite": self.localize("Overwrite"),
+            #l10n: tooltip
+            "overwrite_title": self.localize("Overwrite the current file if it exists"),
+            #l10n: radio button text
+            "merge": self.localize("Merge"),
+            #l10n: tooltip
+            "merge_title": self.localize("Merge the file with the current file and turn conflicts into suggestions"),
+            })
+    return uploadbox
 
   def getchilditems(self, dirfilter):
     """get all the items for directories and files viewable at this level"""
@@ -533,7 +566,7 @@ class ProjectIndex(pagelayout.PootleNavPage):
       depth = 0
     else:
       depth = dirfilter.count(os.path.sep)
-      if not dirfilter.endswith(os.path.extsep + "po"):
+      if not dirfilter.endswith(os.path.extsep + self.project.fileext):
         depth += 1
     diritems = []
     for childdir in self.project.browsefiles(dirfilter=dirfilter, depth=depth, includedirs=True, includefiles=False):
@@ -553,7 +586,7 @@ class ProjectIndex(pagelayout.PootleNavPage):
     """gets the listed dir and fileitems"""
     diritems, fileitems = [], []
     for item in itempaths:
-      if item.endswith(os.path.extsep + "po"):
+      if item.endswith(os.path.extsep + self.project.fileext):
         fileitem = self.getfileitem(item, linksrequired=linksrequired, **newargs)
         fileitems.append((item, fileitem))
       else:
@@ -573,7 +606,7 @@ class ProjectIndex(pagelayout.PootleNavPage):
       depth = 0
     else:
       depth = dirfilter.count(os.path.sep)
-      if not dirfilter.endswith(os.path.extsep + "po"):
+      if not dirfilter.endswith(os.path.extsep + self.project.fileext):
         depth += 1
     allitems = []
     goalchildren = {}
@@ -583,7 +616,7 @@ class ProjectIndex(pagelayout.PootleNavPage):
     for childname in self.project.browsefiles(dirfilter=dirfilter, depth=depth, includedirs=False, includefiles=True):
       allchildren.append(childname)
     initial = dirfilter
-    if initial and not initial.endswith(os.path.extsep + "po"):
+    if initial and not initial.endswith(os.path.extsep + self.project.fileext):
       initial += os.path.sep
     if initial:
       maxdepth = initial.count(os.path.sep)
@@ -676,7 +709,10 @@ class ProjectIndex(pagelayout.PootleNavPage):
   def getfileitem(self, fileentry, linksrequired=None, **newargs):
     """returns an item showing a file entry"""
     if linksrequired is None:
-      linksrequired = ["mine", "review", "quick", "all", "po", "xliff", "ts", "csv", "mo", "update", "commit"]
+      if fileentry.endswith('.po'):
+        linksrequired = ["mine", "review", "quick", "all", "po", "xliff", "ts", "csv", "mo", "update", "commit"]
+      else:
+        linksrequired = ["mine", "review", "quick", "all", "po", "xliff", "update", "commit"]
     basename = os.path.basename(fileentry)
     projectstats = self.project.combinestats([fileentry])
     browseurl = self.getbrowseurl(basename, **newargs)
@@ -684,8 +720,9 @@ class ProjectIndex(pagelayout.PootleNavPage):
     actions = self.getactionlinks(basename, projectstats, linksrequired=linksrequired)
     actionlinks = actions["extended"]
     if "po" in linksrequired:
-      downloadlink = {"href": basename, "text": self.localize('PO file')}
-      actionlinks.append(downloadlink)
+      poname = basename.replace(".xlf", ".po")
+      polink = {"href": poname, "text": self.localize('PO file')}
+      actionlinks.append(polink)
     if "xliff" in linksrequired and "translate" in self.rights:
       xliffname = basename.replace(".po", ".xlf")
       xlifflink = {"href": xliffname, "text": self.localize('XLIFF file')}
