@@ -34,8 +34,10 @@ from Pootle import translatepage
 from Pootle import pagelayout
 from Pootle import projects
 from Pootle import potree
+from Pootle import pootlefile
 from Pootle import users
 from Pootle import filelocations
+from translate.misc import optrecurse
 # Versioning information
 from Pootle import __version__ as pootleversion
 from translate import __version__ as toolkitversion
@@ -46,6 +48,7 @@ from elementtree import ElementTree
 import kid
 import sys
 import os
+import re
 import random
 import pprint
 
@@ -170,9 +173,14 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
             reldirname = dirname.replace(dummyproject.podir, "")
             for fname in fnames:
               fpath = os.path.join(reldirname, fname)
-              if fname.endswith(".po") and not os.path.isdir(os.path.join(dummyproject.podir, fpath)):
+              fullpath = os.path.join(dummyproject.podir, fpath)
+              #TODO: PO specific
+              if fname.endswith(".po") and not os.path.isdir(fullpath):
+                if not os.path.exists(fullpath):
+                  print "file does not exist:", fullpath
+                  return
                 print "refreshing stats for", fpath
-                projects.pootlefile.pootlefile(dummyproject, fpath).updatequickstats()
+                pootlefile.pootlefile(dummyproject, fpath).statistics.updatequickstats()
           os.path.walk(arg, refreshdir, None)
           if projectcode and languagecode:
             dummyproject.savequickstats()
@@ -211,9 +219,15 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
        value = value.decode("utf-8")
       newargdict[key] = value
     argdict = newargdict
-    # TODO: strip off the initial path properly
-    while pathwords and pathwords[0] == "pootle":
+
+    # Strip of the base url
+    baseurl = re.sub('http://[^/]', '', self.instance.baseurl)
+    # Split up and remove empty parts
+    basepathwords = filter(None, baseurl.split('/'))
+    while pathwords and basepathwords and basepathwords[0] == pathwords[0]:
+      basepathwords = basepathwords[1:]
       pathwords = pathwords[1:]
+
     if pathwords:
       top = pathwords[0]
     else:
@@ -241,6 +255,15 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
       csspage.sendfile_path = cssfile
       csspage.allowcaching = True
       return csspage
+    elif top in ['selenium', 'tests']:
+      picturefile = os.path.join(filelocations.htmldir, *pathwords)
+      picture = widgets.SendFile(picturefile)
+      if picturefile.endswith(".html"):
+        picture.content_type = 'text/html'
+      elif picturefile.endswith(".js"):
+        picture.content_type = 'text/javascript'
+      picture.allowcaching = True
+      return picture
     elif top == 'images':
       pathwords = pathwords[1:]
       picturefile = os.path.join(filelocations.htmldir, 'images', *pathwords)
@@ -272,7 +295,7 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
       message = None
       if 'username' in argdict:
         session.username = argdict["username"]
-	message = session.localize("Login failed")
+        message = session.localize("Login failed")
       return users.LoginPage(session, languagenames=self.languagenames, message=message)
     elif top == "register.html":
       return self.registerpage(session, argdict)
@@ -426,18 +449,22 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
             try:
               return translatepage.TranslatePage(project, session, argdict, dirfilter=pofilename)
             except projects.RightsError, stoppedby:
+              if len(pathwords) > 1:
+                dirfilter = os.path.join(*pathwords[:-1])
+              else:
+                dirfilter = ""
               argdict["message"] = str(stoppedby)
-              return indexpage.ProjectIndex(project, session, argdict, dirfilter=pofilename)
+              return indexpage.ProjectIndex(project, session, argdict, dirfilter=dirfilter)
           elif argdict.get("index", 0):
             return indexpage.ProjectIndex(project, session, argdict, dirfilter=pofilename)
           else:
             pofile = project.getpofile(pofilename, freshen=False)
             page = widgets.SendFile(pofile.filename)
             page.etag = str(pofile.pomtime)
-            encoding = pofile.encoding or "UTF-8"
+            encoding = getattr(pofile, "encoding", "UTF-8")
             page.content_type = "text/plain; charset=%s" % encoding
             return page
-        elif bottom.endswith(".csv") or bottom.endswith(".xlf") or bottom.endswith(".ts") or bottom.endswith("mo"):
+        elif bottom.endswith(".csv") or bottom.endswith(".xlf") or bottom.endswith(".ts") or bottom.endswith("po") or bottom.endswith("mo"):
           destfilename = os.path.join(*pathwords)
           basename, extension = os.path.splitext(destfilename)
           pofilename = basename + os.extsep + project.fileext
@@ -451,7 +478,7 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
             page.etag = str(etag)
           else:
             page = widgets.PlainContents(filepath_or_contents)
-          if extension == "csv":
+          if extension == "po" or extension == "csv":
             page.content_type = "text/plain; charset=UTF-8"
           elif extension == "xlf" or extension == "ts":
             page.content_type = "text/xml; charset=UTF-8"
@@ -513,8 +540,8 @@ class PootleOptionParser(simplewebserver.WebOptionParser):
 
 def checkversions():
   """Checks that version dependencies are met"""
-  if not hasattr(toolkitversion, "build") or toolkitversion.build < 9900:
-    raise RuntimeError("requires Translate Toolkit version >= 0.10.  Current installed version is: %s" % toolkitversion.ver)
+  if not hasattr(toolkitversion, "build") or toolkitversion.build < 10000:
+    raise RuntimeError("requires Translate Toolkit version >= 1.0.  Current installed version is: %s" % toolkitversion.ver)
 
 def usepsyco(options):
   # options.psyco == None means the default, which is "full", but don't give a warning...
@@ -523,9 +550,9 @@ def usepsyco(options):
     return
   try:
     import psyco
-  except Exception:
+  except ImportError:
     if options.psyco is not None:
-      self.warning("psyco unavailable", options, sys.exc_info())
+      optrecurse.RecursiveOptionParser(formats={}).warning("psyco unavailable", options, sys.exc_info())
     return
   if options.psyco is None:
     options.psyco = "full"
@@ -542,6 +569,7 @@ def main():
   checkversions()
   parser = PootleOptionParser()
   options, args = parser.parse_args()
+  options.errorlevel = options.logerrors
   usepsyco(options)
   if options.action != "runwebserver":
     options.servertype = "dummy"
