@@ -19,290 +19,87 @@
 # along with translate; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""manages po files and their associated files"""
+"""manages a translation file and its associated files"""
 
+from translate.storage import base
 from translate.storage import po
-from translate.misc import quote
+from translate.storage import xliff
+from translate.storage import factory
 from translate.misc.multistring import multistring
-from translate.filters import checks
 from Pootle import __version__
+from Pootle import statistics
 from jToolkit import timecache
+from jToolkit import glock
 import time
 import os
 
-def getmodtime(filename, default=None):
-  """gets the modificationtime of the given file"""
-  if os.path.exists(filename):
-    return os.stat(filename)[os.path.stat.ST_MTIME]
-  else:
-    return default
+class Wrapper(object):
+  """An object which wraps an inner object, delegating to the encapsulated methods, etc"""
+  def __getattr__(self, attrname, *args):
+    if attrname in self.__dict__:
+      return self.__dict__[attrname]
+    return getattr(self.__dict__["__innerobj__"], attrname, *args)
 
-def wordcount(unquotedstr):
-  """returns the number of words in an unquoted str"""
-  return len(unquotedstr.split())
-
-class pootleelement(po.pounit, object):
-  """a pounit with helpful methods for pootle"""
-  def getunquotedmsgid(self, joinwithlinebreak=False):
-    """returns the msgid as a list of unquoted strings (one per plural form present)"""
-    if self.hasplural():
-      return self.source.strings
-    else:
-      return [self.source]
-  unquotedmsgid = property(getunquotedmsgid)
-
-  def getunquotedmsgstr(self, joinwithlinebreak=False):
-    """returns the msgstr as a list of unquoted strings (one per plural form present)"""
-    if self.hasplural() and isinstance(self.target, multistring):
-      return self.target.strings
-    else:
-      return [self.target]
-
-  def setunquotedmsgstr(self, text):
-    """quotes text in po-style"""
-    if isinstance(text, dict):
-      quotedtext = {}
-      for pluralid in text:
-        pluraltext = text[pluralid].replace("\r\n", "\n")
-        quotedtext[pluralid] = po.quoteforpo(pluraltext)
-      self.msgstr = quotedtext
-    elif isinstance(text, list):
-      if self.hasplural():
-        for i, pluraltext in enumerate(text):
-          if i > len(self.msgstr):
-            self.msgstr.append(po.quoteforpo(pluraltext))
-          else:
-            self.msgstr[i] = po.quoteforpo(pluraltext)
+  def __setattr__(self, attrname, value):
+    if attrname == "__innerobj__":
+      self.__dict__[attrname] = value
+    elif attrname in self.__dict__:
+      if isinstance(self.__dict__[attrname], property):
+        self.__dict__[attrname].fset(value)
       else:
-        if len(text) != 1:
-          raise ValueError("po element has no plural but msgstr has %d elements (%s)" % (len(text), text))
-        self.msgstr = po.quoteforpo(text[0])
+        self.__dict__[attrname] = value
+    elif attrname in self.__class__.__dict__:
+      if isinstance(self.__class__.__dict__[attrname], property):
+        self.__class__.__dict__[attrname].fset(self, value)
+      else:
+        self.__dict__[attrname] = value
     else:
-      text = text.replace("\r\n", "\n")
-      self.msgstr = po.quoteforpo(text)
+      return setattr(self.__dict__["__innerobj__"], attrname, value)
 
-  unquotedmsgstr = property(getunquotedmsgstr, setunquotedmsgstr)
+class LockedFile:
+  """locked interaction with a filesystem file"""
+  #Locking is disabled for now since it impacts performance negatively and was
+  #not complete yet anyway. Reverse svn revision 5271 to regain the locking 
+  #code here.
+  def __init__(self, filename):
+    self.filename = filename
+    self.lock = None
 
-  def classify(self, checker):
-    """returns all classify keys that this element should match, using the checker"""
-    classes = ["total"]
-    if self.isfuzzy():
-      classes.append("fuzzy")
-    if self.isblankmsgstr():
-      classes.append("blank")
-    if not ("fuzzy" in classes or "blank" in classes):
-      classes.append("translated")
-    # TODO: we don't handle checking plurals at all yet, as this is tricky...
-    unquotedid = self.unquotedmsgid[0]
-    unquotedstr = self.unquotedmsgstr[0]
-    if isinstance(unquotedid, str) and isinstance(unquotedstr, unicode):
-      unquotedid = unquotedid.decode(getattr(self, "encoding", "utf-8"))
-    filterresult = checker.run_filters(self, unquotedid, unquotedstr)
-    for filtername, filtermessage in filterresult:
-      classes.append("check-" + filtername)
-    return classes
+  def initlock(self):
+    self.lock = glock.GlobalLock(self.filename + os.extsep + "lock")
 
-class pootlefile(po.pofile):
-  """this represents a pootle-managed .po file and its associated files"""
-  x_generator = "Pootle %s" % __version__.ver
-  def __init__(self, project=None, pofilename=None, stats=True):
-    po.pofile.__init__(self, unitclass=pootleelement)
-    self.pofilename = pofilename
-    if project is None:
-      from Pootle import projects
-      self.project = projects.DummyProject(None)
-      #self.checker = None
-      self.checker = self.project.checker
-      self.filename = self.pofilename
-    else:
-      self.project = project
-      self.checker = self.project.checker
-      self.filename = os.path.join(self.project.podir, self.pofilename)
-    self.statsfilename = self.filename + os.extsep + "stats"
-    self.pendingfilename = self.filename + os.extsep + "pending"
-    self.tmfilename = self.filename + os.extsep + "tm"
-    self.assignsfilename = self.filename + os.extsep + "assigns"
-    self.pendingfile = None
-    # we delay parsing until it is required
-    self.pomtime = None
-    self.classify = {}
-    self.msgidwordcounts = []
-    self.msgstrwordcounts = []
-    if stats:
-      self.getstats()
+  def dellock(self):
+    del self.lock
+    self.lock = None
+
+  def readmodtime(self):
+    """returns the modification time of the file (locked operation)"""
+    return statistics.getmodtime(self.filename)
+
+  def getcontents(self):
+    """returns modtime, contents tuple (locked operation)"""
+    pomtime = statistics.getmodtime(self.filename)
+    fp = open(self.filename, 'r')
+    filecontents = fp.read()
+    fp.close()
+    return pomtime, filecontents
+
+  def writecontents(self, contents):
+    """writes contents to file, returning modification time (locked operation)"""
+    f = open(self.filename, 'w')
+    f.write(contents)
+    f.close()
+    pomtime = statistics.getmodtime(self.filename)
+    return pomtime
+
+class pootleassigns:
+  """this represents the assignments for a file"""
+  def __init__(self, basefile):
+    """constructs assignments object for the given file"""
+    # TODO: try and remove circular references between basefile and this class
+    self.basefile = basefile
+    self.assignsfilename = self.basefile.filename + os.extsep + "assigns"
     self.getassigns()
-    self.tracker = timecache.timecache(20*60)
-
-  def track(self, item, message):
-    """sets the tracker message for the given item"""
-    self.tracker[item] = message
-
-  def readpofile(self):
-    """reads and parses the main po file"""
-    # make sure encoding is reset so it is read from the file
-    self.encoding = None
-    self.units = []
-    pomtime = getmodtime(self.filename)
-    self.parse(open(self.filename, 'r'))
-    # we ignore all the headers by using this filtered set
-    self.transelements = [poel for poel in self.units if not (poel.isheader() or poel.isblank())]
-    self.classifyelements()
-    self.pomtime = pomtime
-
-  def savepofile(self):
-    """saves changes to the main file to disk..."""
-    output = str(self)
-    open(self.filename, "w").write(output)
-    # don't need to reread what we saved
-    self.pomtime = getmodtime(self.filename)
-
-  def pofreshen(self):
-    """makes sure we have a freshly parsed pofile"""
-    if not os.path.exists(self.filename):
-      # the file has been removed, update the project index (and fail below)
-      self.project.scanpofiles()
-    if self.pomtime != getmodtime(self.filename):
-      self.readpofile()
-
-  def getoutput(self):
-    """returns pofile output"""
-    self.pofreshen()
-    return super(pootlefile, self).getoutput()
-
-  def readpendingfile(self):
-    """reads and parses the pending file corresponding to this po file"""
-    if os.path.exists(self.pendingfilename):
-      pendingmtime = getmodtime(self.pendingfilename)
-      if pendingmtime == getattr(self, "pendingmtime", None):
-        return
-      inputfile = open(self.pendingfilename, "r")
-      self.pendingmtime, self.pendingfile = pendingmtime, po.pofile(inputfile, unitclass=self.UnitClass)
-      if self.pomtime:
-        self.reclassifysuggestions()
-    else:
-      self.pendingfile = po.pofile(unitclass=self.UnitClass)
-      self.savependingfile()
-
-  def savependingfile(self):
-    """saves changes to disk..."""
-    output = str(self.pendingfile)
-    open(self.pendingfilename, "w").write(output)
-    self.pendingmtime = getmodtime(self.pendingfilename)
-
-  def readtmfile(self):
-    """reads and parses the tm file corresponding to this po file"""
-    if os.path.exists(self.tmfilename):
-      tmmtime = getmodtime(self.tmfilename)
-      if tmmtime == getattr(self, "tmmtime", None):
-        return
-      inputfile = open(self.tmfilename, "r")
-      self.tmmtime, self.tmfile = tmmtime, po.pofile(inputfile, unitclass=self.UnitClass)
-    else:
-      self.tmfile = po.pofile(unitclass=self.UnitClass)
-
-  def getstats(self):
-    """reads the stats if neccessary or returns them from the cache"""
-    if os.path.exists(self.statsfilename):
-      try:
-        self.readstats()
-      except Exception, e:
-        print "Error reading stats from %s, so recreating (Error was %s)" % (self.statsfilename, e)
-        raise
-        self.statspomtime = None
-    pomtime = getmodtime(self.filename)
-    pendingmtime = getmodtime(self.pendingfilename, None)
-    if hasattr(self, "pendingmtime"):
-      self.readpendingfile()
-    lastpomtime = getattr(self, "statspomtime", None)
-    lastpendingmtime = getattr(self, "statspendingmtime", None)
-    if pomtime is None or pomtime != lastpomtime or pendingmtime != lastpendingmtime:
-      self.calcstats()
-      self.savestats()
-    return self.stats
-
-  def readstats(self):
-    """reads the stats from the associated stats file, setting the required variables"""
-    statsmtime = getmodtime(self.statsfilename)
-    if statsmtime == getattr(self, "statsmtime", None):
-      return
-    stats = open(self.statsfilename, "r").read()
-    mtimes, postatsstring = stats.split("\n", 1)
-    mtimes = mtimes.strip().split()
-    if len(mtimes) == 1:
-      frompomtime = int(mtimes[0])
-      frompendingmtime = None
-    elif len(mtimes) == 2:
-      frompomtime = int(mtimes[0])
-      frompendingmtime = int(mtimes[1])
-    postats = {}
-    msgidwordcounts = []
-    msgstrwordcounts = []
-    for line in postatsstring.split("\n"):
-      if not line.strip():
-        continue
-      if not ":" in line:
-        print "invalid stats line in", self.statsfilename,line
-        continue
-      name, items = line.split(":", 1)
-      if name == "msgidwordcounts":
-        msgidwordcounts = [[int(subitem.strip()) for subitem in item.strip().split("/")] for item in items.strip().split(",") if item]
-      elif name == "msgstrwordcounts":
-        msgstrwordcounts = [[int(subitem.strip()) for subitem in item.strip().split("/")] for item in items.strip().split(",") if item]
-      else:
-        items = [int(item.strip()) for item in items.strip().split(",") if item]
-        postats[name.strip()] = items
-    # save all the read times, data simultaneously
-    self.statspomtime, self.statspendingmtime, self.statsmtime, self.stats, self.msgidwordcounts, self.msgstrwordcounts = frompomtime, frompendingmtime, statsmtime, postats, msgidwordcounts, msgstrwordcounts
-    # if in old-style format (counts instead of items), recalculate
-    totalitems = postats.get("total", [])
-    if len(totalitems) == 1 and totalitems[0] != 0:
-      self.calcstats()
-      self.savestats()
-    if (len(msgidwordcounts) < len(totalitems)) or (len(msgstrwordcounts) < len(totalitems)):
-      self.pofreshen()
-      self.countwords()
-      self.savestats()
-
-  def savestats(self):
-    """saves the current statistics to file"""
-    if not os.path.exists(self.filename):
-      if os.path.exists(self.statsfilename):
-        os.remove(self.statsfilename)
-      return
-    # assumes self.stats is up to date
-    try:
-      postatsstring = "\n".join(["%s:%s" % (name, ",".join(map(str,items))) for name, items in self.stats.iteritems()])
-      wordcountsstring = "msgidwordcounts:" + ",".join(["/".join(map(str,subitems)) for subitems in self.msgidwordcounts])
-      wordcountsstring += "\nmsgstrwordcounts:" + ",".join(["/".join(map(str,subitems)) for subitems in self.msgstrwordcounts])
-      statsfile = open(self.statsfilename, "w")
-      if os.path.exists(self.pendingfilename):
-        statsfile.write("%d %d\n" % (getmodtime(self.filename), getmodtime(self.pendingfilename)))
-      else:
-        statsfile.write("%d\n" % getmodtime(self.filename))
-      statsfile.write(postatsstring + "\n" + wordcountsstring)
-      statsfile.close()
-    except IOError:
-      # TODO: log a warning somewhere. we don't want an error as this is an optimization
-      pass
-    self.updatequickstats()
-
-  def updatequickstats(self):
-    """updates the project's quick stats on this file"""
-    translated = self.stats.get("translated")
-    fuzzy = self.stats.get("fuzzy")
-    translatedwords = sum([sum(self.msgidwordcounts[item]) for item in translated if 0 <= item < len(self.msgidwordcounts)])
-    fuzzywords = sum([sum(self.msgidwordcounts[item]) for item in fuzzy if 0 <= item < len(self.msgidwordcounts)])
-    totalwords = sum([sum(partcounts) for partcounts in self.msgidwordcounts])
-    self.project.updatequickstats(self.pofilename, 
-        translatedwords, len(translated), 
-        fuzzywords, len(fuzzy), 
-        totalwords, len(self.msgidwordcounts))
-
-  def calcstats(self):
-    """calculates translation statistics for the given po file"""
-    self.pofreshen()
-    postats = dict([(name, items) for name, items in self.classify.iteritems()])
-    self.stats = postats
 
   def getassigns(self):
     """reads the assigns if neccessary or returns them from the cache"""
@@ -318,10 +115,12 @@ class pootlefile(po.pofile):
     username: action: itemranges
     where itemranges is a comma-separated list of item numbers or itemranges like 3-5
     e.g.  pootlewizz: review: 2-99,101"""
-    assignsmtime = getmodtime(self.assignsfilename)
+    assignsmtime = statistics.getmodtime(self.assignsfilename)
     if assignsmtime == getattr(self, "assignsmtime", None):
       return
-    assignsstring = open(self.assignsfilename, "r").read()
+    assignsfile = open(self.assignsfilename, "r")
+    assignsstring = assignsfile.read()
+    assignsfile.close()
     poassigns = {}
     itemcount = len(getattr(self, "classify", {}).get("total", []))
     for line in assignsstring.split("\n"):
@@ -414,12 +213,293 @@ class pootlefile(po.pofile):
     assignsfile.writelines(assignstrings)
     assignsfile.close()
 
-  def setmsgstr(self, item, newmsgstr, userprefs, languageprefs):
-    """updates a translation with a new msgstr value"""
+  def getunassigned(self, action=None):
+    """gets all strings that are unassigned (for the given action if given)"""
+    unassigneditems = range(0, self.basefile.statistics.getitemslen())
+    self.assigns = self.getassigns()
+    for username in self.assigns:
+      if action is not None:
+        assigneditems = self.assigns[username].get(action, [])
+      else:
+        assigneditems = []
+        for action, actionitems in self.assigns[username].iteritems():
+          assigneditems += actionitems
+      unassigneditems = [item for item in unassigneditems if item not in assigneditems]
+    return unassigneditems
+
+  def finditems(self, search):
+    """returns items that match the .assignedto and/or .assignedaction criteria in the searchobject"""
+    # search.assignedto == [None] means assigned to nobody
+    if search.assignedto == [None]:
+      assignitems = self.getunassigned(search.assignedaction)
+    else:
+      # filter based on assign criteria
+      assigns = self.getassigns()
+      if search.assignedto:
+        usernames = [search.assignedto]
+      else:
+        usernames = assigns.iterkeys()
+      assignitems = []
+      for username in usernames:
+        if search.assignedaction:
+          actionitems = assigns[username].get(search.assignedaction, [])
+          assignitems.extend(actionitems)
+        else:
+          for actionitems in assigns[username].itervalues():
+            assignitems.extend(actionitems)
+    return assignitems
+
+class pootlefile(Wrapper):
+  """this represents a pootle-managed file and its associated files"""
+  innerclass = po.pofile
+  x_generator = "Pootle %s" % __version__.ver
+  def __init__(self, project=None, pofilename=None, generatestats=True):
+    if pofilename:
+      innerclass = factory.getclass(pofilename)
+    innerobj = innerclass()
+    self.__innerobj__ = innerobj
+    self.UnitClass = innerobj.UnitClass
+    
+    self.pofilename = pofilename
+    if project is None:
+      from Pootle import projects
+      self.project = projects.DummyProject(None)
+      self.checker = None
+      self.filename = self.pofilename
+    else:
+      self.project = project
+      self.checker = self.project.checker
+      self.filename = os.path.join(self.project.podir, self.pofilename)
+    
+    self.lockedfile = LockedFile(self.filename)
+    # we delay parsing until it is required
+    self.pomtime = None
+    self.assigns = pootleassigns(self)
+
+    self.pendingfilename = self.filename + os.extsep + "pending"
+    self.pendingfile = None
+    self.statistics = statistics.pootlestatistics(self, generatestats)
+    self.tmfilename = self.filename + os.extsep + "tm"
+    # we delay parsing until it is required
+    self.pomtime = None
+    self.tracker = timecache.timecache(20*60)
+
+  def __str__(self):
+    return self.__innerobj__.__str__()
+
+  def parsestring(cls, storestring):
+    newstore = cls()
+    newstore.parse(storestring)
+    return newstore
+  parsestring = classmethod(parsestring)
+
+  def parsefile(cls, storefile):
+    """Reads the given file (or opens the given filename) and parses back to an object"""
+    if isinstance(storefile, basestring):
+        storefile = open(storefile, "r")
+    if "r" in getattr(storefile, "mode", "r"):
+      storestring = storefile.read()
+    else:
+      storestring = ""
+    return cls.parsestring(storestring)
+  parsefile = classmethod(parsefile)
+
+  def getheaderplural(self):
+    """returns values for nplural and plural values.  It tries to see if the 
+    file has it specified (in a po header or similar)."""
+    method = getattr(self.__innerobj__, "getheaderplural", None)
+    if method and callable(method):
+      return self.__innerobj__.getheaderplural()
+    else:
+      return None, None
+
+  def updateheaderplural(self, *args, **kwargs):
+    """updates the file header. If there is an updateheader function in the 
+    underlying store it will be delegated there."""
+    method = getattr(self.__innerobj__, "updateheaderplural", None)
+    if method and callable(method):
+      self.__innerobj__.updateheaderplural(*args, **kwargs)
+
+  def updateheader(self, **kwargs):
+    """updates the file header. If there is an updateheader function in the 
+    underlying store it will be delegated there."""
+    method = getattr(self.__innerobj__, "updateheader", None)
+    if method and callable(method):
+      self.__innerobj__.updateheader(**kwargs)
+
+  def readpendingfile(self):
+    """reads and parses the pending file corresponding to this file"""
+    if os.path.exists(self.pendingfilename):
+      pendingmtime = statistics.getmodtime(self.pendingfilename)
+      if pendingmtime == getattr(self, "pendingmtime", None):
+        return
+      inputfile = open(self.pendingfilename, "r")
+      self.pendingmtime, self.pendingfile = pendingmtime, factory.getobject(inputfile, ignore=".pending")
+      if self.pomtime:
+        self.reclassifysuggestions()
+    else:
+      self.pendingfile = po.pofile()
+      self.savependingfile()
+
+  def savependingfile(self):
+    """saves changes to disk..."""
+    output = str(self.pendingfile)
+    outputfile = open(self.pendingfilename, "w")
+    outputfile.write(output)
+    outputfile.close()
+    self.pendingmtime = statistics.getmodtime(self.pendingfilename)
+
+  def readtmfile(self):
+    """reads and parses the tm file corresponding to this file"""
+    if os.path.exists(self.tmfilename):
+      tmmtime = statistics.getmodtime(self.tmfilename)
+      if tmmtime == getattr(self, "tmmtime", None):
+        return
+      inputfile = open(self.tmfilename, "r")
+      self.tmmtime, self.tmfile = tmmtime, factory.getobject(inputfile, ignore=".tm")
+    else:
+      self.tmfile = po.pofile()
+
+  def reclassifysuggestions(self):
+    """shortcut to only update classification of has-suggestion for all items"""
+    suggitems = []
+    sugglocations = {}
+    for thesugg in self.pendingfile.units:
+      locations = tuple(thesugg.getlocations())
+      sugglocations[locations] = thesugg
+    suggitems = [item for item in self.transunits if tuple(item.getlocations()) in sugglocations]
+    havesuggestions = self.statistics.classify["has-suggestion"]
+    for item, poel in enumerate(self.transunits):
+      if (poel in suggitems) != (item in havesuggestions):
+        if poel in suggitems:
+          havesuggestions.append(item)
+        else:
+          havesuggestions.remove(item)
+        havesuggestions.sort()
+    self.statistics.calcstats()
+    self.statistics.savestats()
+
+  def getsuggestions(self, item):
+    """find all the suggestion items submitted for the given item"""
+    unit = self.transunits[item]
+    if isinstance(unit, xliff.xliffunit):
+      return unit.getalttrans()
+
+    locations = unit.getlocations()
+    self.readpendingfile()
+    # TODO: review the matching method
+    suggestpos = [suggestpo for suggestpo in self.pendingfile.units if suggestpo.getlocations() == locations]
+    return suggestpos
+
+  def addsuggestion(self, item, suggtarget, username):
+    """adds a new suggestion for the given item"""
+    unit = self.transunits[item]
+    if isinstance(unit, xliff.xliffunit):
+      if isinstance(suggtarget, list) and (len(suggtarget) > 0):
+        suggtarget = suggtarget[0]
+      unit.addalttrans(suggtarget, origin=username)
+      self.statistics.reclassifyunit(item)
+      self.savepofile()
+      return
+
+    self.readpendingfile()
+    newpo = unit.copy()
+    if username is not None:
+      newpo.msgidcomments.append('"_: suggested by %s\\n"' % username)
+    newpo.target = suggtarget
+    newpo.markfuzzy(False)
+    self.pendingfile.units.append(newpo)
+    self.savependingfile()
+    self.statistics.reclassifyunit(item)
+
+  def deletesuggestion(self, item, suggitem):
+    """removes the suggestion from the pending file"""
+    unit = self.transunits[item]
+    if hasattr(unit, "xmlelement"):
+      suggestions = self.getsuggestions(item)
+      unit.delalttrans(suggestions[suggitem])
+      self.savepofile()
+    else:
+      self.readpendingfile()
+      locations = unit.getlocations()
+      # TODO: remove the suggestion in a less brutal manner
+      pendingitems = [pendingitem for pendingitem, suggestpo in enumerate(self.pendingfile.units) if suggestpo.getlocations() == locations]
+      pendingitem = pendingitems[suggitem]
+      del self.pendingfile.units[pendingitem]
+      self.savependingfile()
+    self.statistics.reclassifyunit(item)
+
+  def getsuggester(self, item, suggitem):
+    """returns who suggested the given item's suggitem if recorded, else None"""
+    unit = self.getsuggestions(item)[suggitem]
+    if hasattr(unit, "xmlelement"):
+      return unit.xmlelement.getAttribute("origin")
+
+    for msgidcomment in unit.msgidcomments:
+      if msgidcomment.find("suggested by ") != -1:
+        suggestedby = po.unquotefrompo([msgidcomment]).replace("_:", "", 1).replace("suggested by ", "", 1).strip()
+        return suggestedby
+    return None
+
+  def gettmsuggestions(self, item):
+    """find all the tmsuggestion items submitted for the given item"""
+    self.readtmfile()
+    unit = self.transunits[item]
+    locations = unit.getlocations()
+    # TODO: review the matching method
+    # Can't simply use the location index, because we want multiple matches
+    suggestpos = [suggestpo for suggestpo in self.tmfile.units if suggestpo.getlocations() == locations]
+    return suggestpos
+
+  def track(self, item, message):
+    """sets the tracker message for the given item"""
+    self.tracker[item] = message
+
+  def readpofile(self):
+    """reads and parses the main file"""
+    # make sure encoding is reset so it is read from the file
+    self.encoding = None
+    self.units = []
+    pomtime, filecontents = self.lockedfile.getcontents()
+    # note: we rely on this not resetting the filename, which we set earlier, when given a string
+    self.parse(filecontents)
+    # we ignore all the headers by using this filtered set
+    self.transunits = [poel for poel in self.units if not (poel.isheader() or poel.isblank())]
+    self.statistics.classifyunits()
+    self.pomtime = pomtime
+
+  def savepofile(self):
+    """saves changes to the main file to disk..."""
+    output = str(self)
+    self.pomtime = self.lockedfile.writecontents(output)
+
+  def pofreshen(self):
+    """makes sure we have a freshly parsed pofile"""
+    if not os.path.exists(self.filename):
+      # the file has been removed, update the project index (and fail below)
+      self.project.scanpofiles()
+    if self.pomtime != self.lockedfile.readmodtime():
+      self.readpofile()
+
+  def getoutput(self):
+    """returns pofile output"""
     self.pofreshen()
-    thepo = self.transelements[item]
-    thepo.unquotedmsgstr = newmsgstr
-    thepo.markfuzzy(False)
+    return super(pootlefile, self).getoutput()
+
+  def updateunit(self, item, newvalues, userprefs, languageprefs):
+    """updates a translation with a new target value"""
+    self.pofreshen()
+    unit = self.transunits[item]
+
+    if newvalues.has_key("target"):
+      unit.target = newvalues["target"]
+    if newvalues.has_key("fuzzy"):
+      unit.markfuzzy(newvalues["fuzzy"])
+    if newvalues.has_key("translator_comments"):
+      unit.removenotes()
+      if newvalues["translator_comments"]:
+        unit.addnote(newvalues["translator_comments"])
+      
     po_revision_date = time.strftime("%F %H:%M%z")
     headerupdates = {"PO_Revision_Date": po_revision_date, "X_Generator": self.x_generator}
     if userprefs:
@@ -432,181 +512,27 @@ class pootlefile(po.pofile):
       if nplurals and pluralequation:
         self.updateheaderplural(nplurals, pluralequation)
     self.savepofile()
-    self.reclassifyelement(item)
-
-  def classifyelements(self):
-    """makes a dictionary of which elements fall into which classifications"""
-    self.classify = {}
-    self.classify["fuzzy"] = []
-    self.classify["blank"] = []
-    self.classify["translated"] = []
-    self.classify["has-suggestion"] = []
-    self.classify["total"] = []
-    for checkname in self.checker.getfilters().keys():
-      self.classify["check-" + checkname] = []
-    for item, poel in enumerate(self.transelements):
-      classes = poel.classify(self.checker)
-      if self.getsuggestions(item):
-        classes.append("has-suggestion")
-      for classname in classes:
-        if classname in self.classify:
-          self.classify[classname].append(item)
-        else:
-          self.classify[classname] = item
-    self.countwords()
-
-  def countwords(self):
-    """counts the words in each of the elements"""
-    self.msgidwordcounts = []
-    self.msgstrwordcounts = []
-    for poel in self.transelements:
-      self.msgidwordcounts.append([wordcount(text) for text in poel.unquotedmsgid])
-      self.msgstrwordcounts.append([wordcount(text) for text in poel.unquotedmsgstr])
-
-  def reclassifyelement(self, item):
-    """updates the classification of poel in self.classify"""
-    poel = self.transelements[item]
-    self.msgidwordcounts[item] = [wordcount(text) for text in poel.unquotedmsgid]
-    self.msgstrwordcounts[item] = [wordcount(text) for text in poel.unquotedmsgstr]
-    classes = poel.classify(self.checker)
-    if self.getsuggestions(item):
-      classes.append("has-suggestion")
-    for classname, matchingitems in self.classify.items():
-      if (classname in classes) != (item in matchingitems):
-        if classname in classes:
-          self.classify[classname].append(item)
-        else:
-          self.classify[classname].remove(item)
-        self.classify[classname].sort()
-    self.calcstats()
-    self.savestats()
-
-  def reclassifysuggestions(self):
-    """shortcut to only update classification of has-suggestion for all items"""
-    suggitems = []
-    sugglocations = {}
-    for thesugg in self.pendingfile.units:
-      locations = tuple(thesugg.getlocations())
-      sugglocations[locations] = thesugg
-    suggitems = [item for item in self.transelements if tuple(item.getlocations()) in sugglocations]
-    havesuggestions = self.classify["has-suggestion"]
-    for item, poel in enumerate(self.transelements):
-      if (poel in suggitems) != (item in havesuggestions):
-        if poel in suggitems:
-          havesuggestions.append(item)
-        else:
-          havesuggestions.remove(item)
-        havesuggestions.sort()
-    self.calcstats()
-    self.savestats()
-
-  def getsuggestions(self, item):
-    """find all the suggestion items submitted for the given (pofile or pofilename) and item"""
-    self.readpendingfile()
-    thepo = self.transelements[item]
-    locations = thepo.getlocations()
-    # TODO: review the matching method
-    suggestpos = [suggestpo for suggestpo in self.pendingfile.units if suggestpo.getlocations() == locations]
-    return suggestpos
-
-  def addsuggestion(self, item, suggmsgstr, username):
-    """adds a new suggestion for the given item to the pendingfile"""
-    self.readpendingfile()
-    thepo = self.transelements[item]
-    newpo = thepo.copy()
-    if username is not None:
-      newpo.msgidcomments.append('"_: suggested by %s\\n"' % username)
-    newpo.unquotedmsgstr = suggmsgstr
-    newpo.markfuzzy(False)
-    self.pendingfile.units.append(newpo)
-    self.savependingfile()
-    self.reclassifyelement(item)
-
-  def deletesuggestion(self, item, suggitem):
-    """removes the suggestion from the pending file"""
-    self.readpendingfile()
-    thepo = self.transelements[item]
-    locations = thepo.getlocations()
-    # TODO: remove the suggestion in a less brutal manner
-    pendingitems = [pendingitem for pendingitem, suggestpo in enumerate(self.pendingfile.units) if suggestpo.getlocations() == locations]
-    pendingitem = pendingitems[suggitem]
-    del self.pendingfile.units[pendingitem]
-    self.savependingfile()
-    self.reclassifyelement(item)
-
-  def gettmsuggestions(self, item):
-    """find all the tmsuggestion items submitted for the given item"""
-    self.readtmfile()
-    thepo = self.transelements[item]
-    locations = thepo.getlocations()
-    # TODO: review the matching method
-    # Can't simply use the location index, because we want multiple matches
-    suggestpos = [suggestpo for suggestpo in self.tmfile.units if suggestpo.getlocations() == locations]
-    return suggestpos
-
-  def getitemslen(self):
-    """gets the number of items in the file"""
-    # TODO: simplify this, and use wherever its needed
-    if hasattr(self, "transelements"):
-      return len(self.transelements)
-    elif hasattr(self, "stats") and "total" in self.stats:
-      return len(self.stats["total"])
-    elif hasattr(self, "classify") and "total" in self.classify:
-      return len(self.classify["total"])
-    else:
-      # we hadn't read stats...
-      return len(self.getstats()["total"])
-
-  def getunassigned(self, action=None):
-    """gets all strings that are unassigned (for the given action if given)"""
-    unassigneditems = range(0, self.getitemslen())
-    assigns = self.getassigns()
-    for username in self.assigns:
-      if action is not None:
-        assigneditems = self.assigns[username].get(action, [])
-      else:
-        assigneditems = []
-        for action, actionitems in self.assigns[username].iteritems():
-          assigneditems += actionitems
-      unassigneditems = [item for item in unassigneditems if item not in assigneditems]
-    return unassigneditems
+    self.statistics.reclassifyunit(item)
 
   def iteritems(self, search, lastitem=None):
     """iterates through the items in this pofile starting after the given lastitem, using the given search"""
     # update stats if required
-    self.getstats()
+    self.statistics.getstats()
     if lastitem is None:
       minitem = 0
     else:
       minitem = lastitem + 1
-    maxitem = len(self.transelements)
+    maxitem = len(self.transunits)
     validitems = range(minitem, maxitem)
-    if search.assignedto or search.assignedaction: 
-      # search.assignedto == [None] means assigned to nobody
-      if search.assignedto == [None]:
-        assignitems = self.getunassigned(search.assignedaction)
-      else:
-        # filter based on assign criteria
-        self.getassigns()
-        if search.assignedto:
-          usernames = [search.assignedto]
-        else:
-          usernames = self.assigns.iterkeys()
-        assignitems = []
-        for username in usernames:
-          if search.assignedaction:
-            actionitems = self.assigns[username].get(search.assignedaction, [])
-            assignitems.extend(actionitems)
-          else:
-            for actionitems in self.assigns[username].itervalues():
-              assignitems.extend(actionitems)
+    if search.assignedto or search.assignedaction:
+      assignitems = self.assigns.finditems(search)
       validitems = [item for item in validitems if item in assignitems]
     # loop through, filtering on matchnames if required
     for item in validitems:
       if not search.matchnames:
         yield item
       for name in search.matchnames:
-        if item in self.classify[name]:
+        if item in self.statistics.classify[name]:
           yield item
 
   def matchitems(self, newfile, uselocations=False):
@@ -636,9 +562,9 @@ class pootlefile(po.pofile):
       if not foundid:
         # We can't use the multistring, because it might contain more than two
         # entries in a PO xliff file. Rather use the singular.
-        msgid = unicode(newpo.source)
-        if msgid in self.sourceindex:
-          oldpo = self.sourceindex[msgid]
+        source = unicode(newpo.source)
+        if source in self.sourceindex:
+          oldpo = self.sourceindex[source]
           matches.append((oldpo, newpo))
         else:
           matches.append((None, newpo))
@@ -656,7 +582,7 @@ class pootlefile(po.pofile):
     if not oldpo.target or not newpo.target or oldpo.isheader() or newpo.isheader() or unchanged:
       oldpo.merge(newpo)
     else:
-      for item, matchpo in enumerate(self.transelements):
+      for item, matchpo in enumerate(self.transunits):
         if matchpo == oldpo:
           strings = getattr(newpo.target, "strings", [newpo.target])
           self.addsuggestion(item, strings, username)
@@ -702,7 +628,7 @@ class pootlefile(po.pofile):
     headerstoaccept = {}
     ownheader = self.parseheader()
     for (key, value) in newfile.parseheader().items():
-      if key in updatekeys or (not key in ownheader or not ownheader[key]) and key in self.header_order:
+      if key in updatekeys or (not key in ownheader or not ownheader[key]) and key in po.pofile.header_order:
         headerstoaccept[key] = value
     self.updateheader(add=True, **headerstoaccept)
     
@@ -712,7 +638,7 @@ class pootlefile(po.pofile):
     if header is None and not newheader is None:
       header = self.UnitClass("", encoding=self.encoding)
       header.target = ""
-    if header:  
+    if header:
       header.initallcomments(blankall=True)
       if newheader:
         for i in range(len(header.allcomments)):
@@ -722,6 +648,22 @@ class pootlefile(po.pofile):
     # the easiest way to recalculate everything
     self.readpofile()
 
+  def getfilename(self, path=None):
+    """Checks for and returns filename or path of pootlefile"""
+    if not os.path.exists(self.filename):
+      return None
+    if path:
+      return self.filename
+    else:
+      return self.filename.split("/")[-1]
+
+  def getext(self):
+    """Returns the extension of the pootlefile"""
+    if not os.path.exists(self.filename):
+      return None
+    else:
+      return os.path.splitext(self.filename)[1]
+
 class Search:
   """an object containing all the searching information"""
   def __init__(self, dirfilter=None, matchnames=[], assignedto=None, assignedaction=None, searchtext=None):
