@@ -21,18 +21,19 @@
 
 """manages projects and files and translations"""
 
-from translate.storage import po
 from translate.storage import factory
 from translate.filters import checks
 from translate.filters import pofilter
 from translate.convert import po2csv
 from translate.convert import po2xliff
+from translate.convert import xliff2po
 from translate.convert import po2ts
 from translate.convert import pot2po
 from translate.convert import po2oo
 from translate.tools import pocompile
 from translate.tools import pogrep
 from translate.search import match
+from Pootle import statistics
 from Pootle import pootlefile
 from Pootle import versioncontrol
 from jToolkit import timecache
@@ -43,7 +44,6 @@ import cStringIO
 import traceback
 import gettext
 from jToolkit.data import indexer
-from jToolkit import glock
 
 class RightsError(ValueError):
   pass
@@ -61,7 +61,7 @@ class InternalAdminSession:
     return True
 
 class potimecache(timecache.timecache):
-  """caches pootlefile objects, remembers time, and reverts back to statistics when neccessary..."""
+  """Caches pootlefile objects, remembers time, and reverts back to statistics when necessary..."""
   def __init__(self, expiryperiod, project):
     """initialises the cache to keep objects for the given expiryperiod, and point back to the project"""
     timecache.timecache.__init__(self, expiryperiod)
@@ -97,7 +97,8 @@ class TranslationProject(object):
     self.pofiles = potimecache(15*60, self)
     self.projectcheckerstyle = self.potree.getprojectcheckerstyle(self.projectcode)
     checkerclasses = [checks.projectcheckers.get(self.projectcheckerstyle, checks.StandardChecker), pofilter.StandardPOChecker]
-    self.checker = pofilter.POTeeChecker(checkerclasses=checkerclasses, errorhandler=self.filtererrorhandler)
+    self.checker = pofilter.POTeeChecker(checkerclasses=checkerclasses, errorhandler=self.filtererrorhandler, languagecode=languagecode)
+    self.fileext = self.potree.getprojectlocalfiletype(self.projectcode)
     self.quickstats = {}
     # terminology matcher
     self.termmatcher = None
@@ -135,10 +136,12 @@ class TranslationProject(object):
     return [("view", localize("View")),
             ("suggest", localize("Suggest")),
             ("translate", localize("Translate")),
+            ("overwrite", localize("Overwrite")),
             # l10n: Verb
             ("review", localize("Review")),
             # l10n: Verb
             ("archive", localize("Archive")),
+            # l10n: This refers to generating the binary .mo file
             ("pocompile", localize("Compile PO files")),
             ("assign", localize("Assign")),
             ("admin", localize("Administrate")),
@@ -170,7 +173,7 @@ class TranslationProject(object):
           if self.languagecode == "en":
             rights = "view, archive, pocompile"
           else:
-            rights = "view, review, translate, archive, pocompile"
+            rights = self.potree.getdefaultrights()
         else:
           rights = getattr(rightstree, "default", None)
       else:
@@ -185,7 +188,7 @@ class TranslationProject(object):
     """returns all the users who registered for this language and project"""
 
     def usableuser(user, userprefs):
-      if username in ["__dummy__", "default", "nobody"]:
+      if user in ["__dummy__", "default", "nobody"]:
         return False
       return self.languagecode in getattr(userprefs, "languages", [])
 
@@ -193,7 +196,7 @@ class TranslationProject(object):
     for username, userprefs in session.loginchecker.users.iteritems():
       if usableuser(username, userprefs):
         # Let's build a nice descriptive name for use in the interface. It will
-        # contain both the usernaame and the full name, if available.
+        # contain both the username and the full name, if available.
         name = getattr(userprefs, "name", None)
         if name:
           description = "%s (%s)" % (name, username)
@@ -251,7 +254,7 @@ class TranslationProject(object):
     expanddirs specifies whether to expand directories and return all files in them
     includepartial specifies whether to return directories that are not in the goal, but have files below maxdepth in the goal"""
     if not goalname:
-      return self.getnogoalfiles(dirfilter=dirfilter, maxdepth=maxdepth, includedirs=includedirs, expanddirs=expanddirs, includepartial=includepartial)
+      return self.getnogoalfiles(dirfilter=dirfilter, maxdepth=maxdepth, includedirs=includedirs, expanddirs=expanddirs)
     goals = getattr(self.prefs, "goals", {})
     poext = os.extsep + self.fileext
     pathsep = os.path.sep
@@ -261,7 +264,7 @@ class TranslationProject(object):
       goalmembers = getattr(goalnode, "files", "")
       goalmembers = [goalfile.strip() for goalfile in goalmembers.split(",") if goalfile.strip()]
       goaldirs = [goaldir for goaldir in goalmembers if goaldir.endswith(pathsep)]
-      goalfiles = [goalfile for goalfile in goalmembers if not goalfile.endswith(pathsep)]
+      goalfiles = [goalfile for goalfile in goalmembers if not goalfile.endswith(pathsep) and goalfile in self.pofilenames]
       if expanddirs:
         expandgoaldirs = []
         expandgoalfiles = []
@@ -292,12 +295,11 @@ class TranslationProject(object):
         return unique(goalfiles)
     return []
 
-  def getnogoalfiles(self, dirfilter=None, maxdepth=None, includedirs=True, expanddirs=False, includepartial=False):
+  def getnogoalfiles(self, dirfilter=None, maxdepth=None, includedirs=True, expanddirs=False):
     """Returns the files that are not in a goal. This works with getgoalfiles
     and therefre the API resembles that closely"""
     all = self.browsefiles(dirfilter=dirfilter, maxdepth=maxdepth, includedirs=includedirs)
     pathsep = os.path.sep
-    goals = getattr(self.prefs, "goals", {})
     for testgoalname in self.getgoals():
       goalfiles = self.getgoalfiles(testgoalname, dirfilter=dirfilter, maxdepth=maxdepth, includedirs=includedirs, expanddirs=expanddirs, includepartial=False)
       for goalfile in goalfiles:
@@ -460,46 +462,51 @@ class TranslationProject(object):
       if not pofilename in self.pofilenames:
         del self.pofiles[pofilename]
 
-  def getuploadpath(self, dirname, pofilename):
-    """gets the path of a po file being uploaded securely, creating directories as neccessary"""
+  def getuploadpath(self, dirname, localfilename):
+    """gets the path of a translation file being uploaded securely, creating directories as neccessary"""
     if os.path.isabs(dirname) or dirname.startswith("."):
       raise ValueError("invalid/insecure file path: %s" % dirname)
-    if os.path.basename(pofilename) != pofilename or pofilename.startswith("."):
-      raise ValueError("invalid/insecure file name: %s" % pofilename)
+    if os.path.basename(localfilename) != localfilename or localfilename.startswith("."):
+      raise ValueError("invalid/insecure file name: %s" % localfilename)
     if self.filestyle == "gnu":
-      if not self.potree.languagematch(self.languagecode, pofilename[:-len(".po")]):
-        raise ValueError("invalid GNU-style file name %s: must match '%s.po' or '%s[_-][A-Z]{2,3}.po'" % (pofilename, self.languagecode, self.languagecode))
+      if not self.potree.languagematch(self.languagecode, localfilename[:-len("."+self.fileext)]):
+        raise ValueError("invalid GNU-style file name %s: must match '%s.%s' or '%s[_-][A-Z]{2,3}.%s'" % (localfilename, self.fileext, self.languagecode, self.languagecode, self.fileext))
     dircheck = self.podir
     for part in dirname.split(os.sep):
       dircheck = os.path.join(dircheck, part)
       if dircheck and not os.path.isdir(dircheck):
         os.mkdir(dircheck)
-    return os.path.join(self.podir, dirname, pofilename)
+    return os.path.join(self.podir, dirname, localfilename)
 
-  def uploadfile(self, session, dirname, filename, contents):
+  def uploadfile(self, session, dirname, filename, contents, overwrite=False):
     """uploads an individual file"""
     pathname = self.getuploadpath(dirname, filename)
-    for extention in ["xliff", "xlf"]:
+    for extention in ["xliff", "xlf", "xlff"]:
       if filename.endswith(extention):
-        pofilename = filename[:-len(os.extsep+extention)] + os.extsep + "po"
+        pofilename = filename[:-len(os.extsep+extention)] + os.extsep + self.fileext
         popathname = self.getuploadpath(dirname, pofilename)
         break
     else:
       pofilename = filename
       popathname = pathname
-    if os.path.exists(popathname):
+
+    rights = self.getrights(session)
+
+    if os.path.exists(popathname) and not overwrite:
       origpofile = self.getpofile(os.path.join(dirname, pofilename))
       newfileclass = factory.getclass(pathname)
       infile = cStringIO.StringIO(contents)
       newfile = newfileclass.parsefile(infile)
-      if "admin" in self.getrights(session):
+      if "admin" in rights:
         origpofile.mergefile(newfile, session.username)
-      elif "translate" in self.getrights(session):
+      elif "translate" in rights:
         origpofile.mergefile(newfile, session.username, allownewstrings=False)
       else:
         raise RightsError(session.localize("You do not have rights to upload files here"))
     else:
-      if "admin" not in self.getrights(session):
+      if overwrite and not ("admin" in rights or "overwrite" in rights):
+        raise RightsError(session.localize("You do not have rights to overwrite files here"))
+      elif not os.path.exists(popathname) and not ("admin" in rights or "overwrite" in rights):
         raise RightsError(session.localize("You do not have rights to upload new files here"))
       outfile = open(popathname, "wb")
       outfile.write(contents)
@@ -543,19 +550,20 @@ class TranslationProject(object):
           localmsgstr = localpo.target
           if origmsgstr == localmsgstr:
             continue
+
         foundsource = False
-        if usesources:
-          for location in origpo.getlocations():
-            if location in newpofile.locationindex:
-              newpo = newpofile.locationindex[location]
-              if newpo is not None:
-                foundsource = True
-                newmatches.append((newpo, localpo))
-                continue
+        # First try to find a match on location
+        for location in origpo.getlocations():
+          if location in newpofile.locationindex:
+            newpo = newpofile.locationindex[location]
+            if newpo is not None and newpo.source == localpo.source:
+              foundsource = True
+              newmatches.append((newpo, localpo))
+              continue
         if not foundsource:
-          msgid = origpo.unquotedmsgid
-          if msgid in newpofile.sourceindex:
-            newpo = newpofile.sourceindex[msgid]
+          source = origpo.source
+          if source in newpofile.sourceindex:
+            newpo = newpofile.sourceindex[source]
             newmatches.append((newpo, localpo))
           else:
             newmatches.append((None, localpo))
@@ -617,6 +625,7 @@ class TranslationProject(object):
         pofilename = self.languagecode + os.extsep + "po"
       else:
         pofilename = potfilename[:-len(os.extsep+"pot")] + os.extsep + "po"
+        pofilename = os.path.basename(pofilename)
       origpofilename = os.path.join(self.podir, pofilename)
       if os.path.exists(origpofilename):
         origpofile = open(origpofilename)
@@ -765,10 +774,9 @@ class TranslationProject(object):
     """updates the index with the contents of pofilename (limit to items if given)"""
     if not indexer.HAVE_INDEXER:
       return
-    needsupdate = True
     pofile = self.pofiles[pofilename]
     # check if the pomtime in the index == the latest pomtime
-    pomtime = pootlefile.getmodtime(pofile.filename)
+    pomtime = statistics.getmodtime(pofile.filename)
     pofilenamequery = self.searcher.makeQuery([("pofilename", pofilename)], True)
     pomtimequery = self.searcher.makeQuery([("pomtime", str(pomtime))], True)
     if items is not None:
@@ -777,7 +785,7 @@ class TranslationProject(object):
     gooditems = self.searcher.search(gooditemsquery, "itemno")
     allitems = self.searcher.search(pofilenamequery, "itemno")
     if items is None:
-      if len(gooditems) == len(allitems) == pofile.getitemslen():
+      if len(gooditems) == len(allitems) == pofile.statistics.getitemslen():
         return
       print "updating", self.projectcode, self.languagecode, "index for", pofilename
       self.searcher.deleteDoc({"pofilename": pofilename})
@@ -787,16 +795,12 @@ class TranslationProject(object):
     pofile.pofreshen()
     addlist = []
     if items is None:
-      items = range(len(pofile.transelements))
+      items = range(len(pofile.transunits))
     for itemno in items:
-      thepo = pofile.transelements[itemno]
+      unit = pofile.transunits[itemno]
       doc = {"pofilename": pofilename, "pomtime": str(pomtime), "itemno": str(itemno)}
-      if thepo.hasplural():
-        orig = "\n".join(thepo.source.strings)
-        trans = "\n".join(thepo.target.strings)
-      else:
-        orig = thepo.source
-        trans = thepo.target
+      orig = "\n".join(unit.source.strings)
+      trans = "\n".join(unit.target.strings)
       doc["msgid"] = orig
       doc["msgstr"] = trans
       addlist.append(doc)
@@ -814,9 +818,9 @@ class TranslationProject(object):
     # search.assignedto == [None] means assigned to nobody
     if search.assignedto or search.assignedaction:
       if search.assignedto == [None]:
-        assigns = self.pofiles[pofilename].getunassigned(search.assignedaction)
+        assigns = self.pofiles[pofilename].assigns.getunassigned(search.assignedaction)
       else:
-        assigns = self.pofiles[pofilename].getassigns()
+        assigns = self.pofiles[pofilename].assigns.getassigns()
         if search.assignedto is not None:
           if search.assignedto not in assigns:
             return False
@@ -873,10 +877,10 @@ class TranslationProject(object):
   def searchpoitems(self, pofilename, item, search):
     """finds the next item matching the given search"""
     if search.searchtext:
-      pogrepfilter = pogrep.pogrepfilter(search.searchtext, None, ignorecase=True)
+      grepfilter = pogrep.GrepFilter(search.searchtext, None, ignorecase=True)
     for pofilename in self.searchpofilenames(pofilename, search, includelast=True):
       pofile = self.getpofile(pofilename)
-      if indexer.HAVE_INDEXER:
+      if indexer.HAVE_INDEXER and (search.searchtext or search.matchnames):
         filesearch = search.copy()
         filesearch.dirfilter = pofilename
         hits = self.indexsearch(filesearch, "itemno")
@@ -895,8 +899,8 @@ class TranslationProject(object):
             continue
         # TODO: move this to iteritems
         if search.searchtext:
-          thepo = pofile.transelements[item]
-          if pogrepfilter.filterelement(thepo):
+          unit = pofile.transunits[item]
+          if grepfilter.filterelement(unit):
             yield pofilename, item
         else:
           yield pofilename, item
@@ -914,14 +918,14 @@ class TranslationProject(object):
     if not "assign" in self.getrights(session):
       raise RightsError(session.localize("You do not have rights to alter assignments here"))
     if search.searchtext:
-      pogrepfilter = pogrep.pogrepfilter(search.searchtext, None, ignorecase=True)
+      grepfilter = pogrep.GrepFilter(search.searchtext, None, ignorecase=True)
     if not isinstance(assignto, list):
       assignto = [assignto]
     usercount = len(assignto)
     assigncount = 0
     if not usercount:
       return assigncount
-    docountwords = lambda pofilename: self.countwords([(pofilename, item) for item in range(self.pofiles[pofilename].getitemslen())])
+    docountwords = lambda pofilename: self.countwords([(pofilename, item) for item in range(self.pofiles[pofilename].statistics.getitemslen())])
     pofilenames = [pofilename for pofilename in self.searchpofilenames(None, search, includelast=True)]
     wordcounts = [(pofilename, docountwords(pofilename)) for pofilename in pofilenames]
     totalwordcount = sum([wordcount for pofilename, wordcount in wordcounts])
@@ -936,8 +940,8 @@ class TranslationProject(object):
         # TODO: move this to iteritems
         if search.searchtext:
           validitem = False
-          thepo = pofile.transelements[item]
-          if pogrepfilter.filterelement(thepo):
+          unit = pofile.transunits[item]
+          if grepfilter.filterelement(unit):
             validitem = True
           if not validitem:
             continue
@@ -946,7 +950,7 @@ class TranslationProject(object):
           usernum = min(usernum+1, len(assignto)-1)
           userwords = 0
         userwords += itemwordcount
-        pofile.assignto(item, assignto[usernum], action)
+        pofile.assigns.assignto(item, assignto[usernum], action)
         assigncount += 1
     return assigncount
 
@@ -955,19 +959,19 @@ class TranslationProject(object):
     if not "assign" in self.getrights(session):
       raise RightsError(session.localize("You do not have rights to alter assignments here"))
     if search.searchtext:
-      pogrepfilter = pogrep.pogrepfilter(search.searchtext, None, ignorecase=True)
+      grepfilter = pogrep.GrepFilter(search.searchtext, None, ignorecase=True)
     assigncount = 0
     for pofilename in self.searchpofilenames(None, search, includelast=True):
       pofile = self.getpofile(pofilename)
       for item in pofile.iteritems(search, None):
         # TODO: move this to iteritems
         if search.searchtext:
-          thepo = pofile.transelements[item]
-          if pogrepfilter.filterelement(thepo):
-            pofile.unassign(item, assignedto, action)
+          unit = pofile.transunits[item]
+          if grepfilter.filterelement(unit):
+            pofile.assigns.unassign(item, assignedto, action)
             assigncount += 1
         else:
-          pofile.unassign(item, assignedto, action)
+          pofile.assigns.unassign(item, assignedto, action)
           assigncount += 1
     return assigncount
 
@@ -997,7 +1001,7 @@ class TranslationProject(object):
         items = line.split(",")
         if len(items) != 7:
           #Must be an old format style without the fuzzy stats
-          quickstats = self.getquickstats()
+          self.quickstats = self.getquickstats()
           self.savequickstats()
           break
         else:
@@ -1006,7 +1010,7 @@ class TranslationProject(object):
               translatedwords, translated, fuzzywords, fuzzy, totalwords, total])
 
   def getquickstats(self, pofilenames=None):
-    """gets translated and total stats and wordcouts without doing calculations returning dictionary"""
+    """Gets translated and total stats and wordcounts without doing calculations returning dictionary."""
     if pofilenames is None:
       pofilenames = self.pofilenames
     alltranslatedwords, alltranslated, allfuzzywords, allfuzzy, alltotalwords, alltotal = 0, 0, 0, 0, 0, 0
@@ -1023,7 +1027,7 @@ class TranslationProject(object):
       alltotalwords += totalwords
       alltotal += total
     for pofilename in slowfiles:
-      self.pofiles[pofilename].updatequickstats()
+      self.pofiles[pofilename].statistics.updatequickstats()
       translatedwords, translated, fuzzywords, fuzzy, totalwords, total = self.quickstats[pofilename]
       alltranslatedwords += translatedwords
       alltranslated += translated
@@ -1066,8 +1070,8 @@ class TranslationProject(object):
     wordcount = 0
     for pofilename, item in stats:
       pofile = self.pofiles[pofilename]
-      if 0 <= item < len(pofile.msgidwordcounts):
-        wordcount += sum(pofile.msgidwordcounts[item])
+      if 0 <= item < len(pofile.statistics.sourcewordcounts):
+        wordcount += sum(pofile.statistics.sourcewordcounts[item])
     return wordcount
 
   def getpomtime(self):
@@ -1096,12 +1100,12 @@ class TranslationProject(object):
 
   def getpostats(self, pofilename):
     """calculates translation statistics for the given po file"""
-    return self.pofiles[pofilename].getstats()
+    return self.pofiles[pofilename].statistics.getstats()
 
   def getassignstats(self, pofilename, action=None):
     """calculates translation statistics for the given po file (can filter by action if given)"""
     polen = len(self.getpostats(pofilename)["total"])
-    assigns = self.pofiles[pofilename].getassigns()
+    assigns = self.pofiles[pofilename].assigns.getassigns()
     assignstats = {}
     for username, userassigns in assigns.iteritems():
       allitems = []
@@ -1123,45 +1127,22 @@ class TranslationProject(object):
     """returns number of items in the given pofilename"""
     # TODO: needn't parse the file for this ...
     pofile = self.getpofile(pofilename)
-    return len(pofile.transelements)
-
-  def getitem(self, pofilename, item):
-    """returns a particular item from a particular po file's orig, trans strings as a tuple"""
-    pofile = self.getpofile(pofilename)
-    thepo = pofile.transelements[item]
-    orig, trans = self.source, self.target
-    return orig, trans
-
-  def getitemclasses(self, pofilename, item):
-    """returns which classes this item belongs to"""
-    # TODO: needn't parse the file for this ...
-    pofile = self.getpofile(pofilename)
-    return [classname for (classname, classitems) in pofile.classify.iteritems() if item in classitems]
-
-  def unquotefrompo(self, postr):
-    """extracts a po-quoted string to normal text"""
-    if isinstance(postr, dict):
-      quotedpo = {}
-      for pluralid in postr:
-        quotedpo[pluralid] = self.unquotefrompo(postr[pluralid])
-      return quotedpo
-    else:
-      return po.unquotefrompo(postr)
+    return len(pofile.transunits)
 
   def getitems(self, pofilename, itemstart, itemstop):
     """returns a set of items from the pofile, converted to original and translation strings"""
     pofile = self.getpofile(pofilename)
-    elements = pofile.transelements[max(itemstart,0):itemstop]
-    return elements
+    units = pofile.transunits[max(itemstart,0):itemstop]
+    return units
 
-  def updatetranslation(self, pofilename, item, trans, session):
+  def updatetranslation(self, pofilename, item, newvalues, session):
     """updates a translation with a new value..."""
     if "translate" not in self.getrights(session):
       raise RightsError(session.localize("You do not have rights to change translations here"))
     pofile = self.pofiles[pofilename]
     pofile.track(item, "edited by %s" % session.username)
     languageprefs = getattr(session.instance.languages, self.languagecode, None)
-    pofile.setmsgstr(item, trans, session.prefs, languageprefs)
+    pofile.updateunit(item, newvalues, session.prefs, languageprefs)
     self.updateindex(pofilename, [item])
 
   def suggesttranslation(self, pofilename, item, trans, session):
@@ -1189,19 +1170,14 @@ class TranslationProject(object):
       pofile = self.getpofile(pofilename)
     pofile.track(item, "suggestion by %s accepted by %s" % (self.getsuggester(pofile, item, suggitem), session.username))
     pofile.deletesuggestion(item, suggitem)
-    self.updatetranslation(pofilename, item, newtrans, session)
+    self.updatetranslation(pofilename, item, {"target": newtrans, "fuzzy": False}, session)
 
   def getsuggester(self, pofile, item, suggitem):
     """returns who suggested the given item's suggitem if recorded, else None"""
     if isinstance(pofile, (str, unicode)):
       pofilename = pofile
       pofile = self.getpofile(pofilename)
-    suggestionpo = pofile.getsuggestions(item)[suggitem]
-    for msgidcomment in suggestionpo.msgidcomments:
-      if msgidcomment.find("suggested by ") != -1:
-        suggestedby = po.unquotefrompo([msgidcomment]).replace("_:", "", 1).replace("suggested by ", "", 1).strip()
-        return suggestedby
-    return None
+    return pofile.getsuggester(item, suggitem)
 
   def rejectsuggestion(self, pofile, item, suggitem, newtrans, session):
     """rejects the suggestion and removes it from the pending file"""
@@ -1234,7 +1210,7 @@ class TranslationProject(object):
           termfile.pofreshen()
         return self
     else:
-      termfilename = "pootle-terminology.po"
+      termfilename = "pootle-terminology."+self.fileext
       if termfilename in self.pofiles:
         termfile = self.getpofile(termfilename, freshen=True)
         return termfile
@@ -1269,13 +1245,13 @@ class TranslationProject(object):
       if isinstance(pofile, (str, unicode)):
         pofilename = pofile
         pofile = self.getpofile(pofilename)
-        return termmatcher.matches(pofile.transelements[item].source)
+        return termmatcher.matches(pofile.transunits[item].source)
     except Exception, e:
       session.server.errorhandler.logerror(traceback.format_exc())
       return []
 
   def savepofile(self, pofilename):
-    """saves changes to disk..."""
+    """saves changes to disk"""
     pofile = self.getpofile(pofilename)
     pofile.savepofile()
 
@@ -1286,18 +1262,17 @@ class TranslationProject(object):
 
   def convert(self, pofilename, destformat):
     """converts the pofile to the given format, returning (etag_if_filepath, filepath_or_contents)"""
-    destfilename = pofilename[:-len(self.fileext)] + destformat
     pofile = self.getpofile(pofilename, freshen=False)
     destfilename = pofile.filename[:-len(self.fileext)] + destformat
-    destmtime = pootlefile.getmodtime(destfilename)
-    pomtime = pootlefile.getmodtime(pofile.filename)
+    destmtime = statistics.getmodtime(destfilename)
+    pomtime = statistics.getmodtime(pofile.filename)
     if pomtime and destmtime == pomtime:
       try:
         return pomtime, destfilename
       except Exception, e:
         print "error reading cached converted file %s: %s" % (destfilename, e)
     pofile.pofreshen()
-    converters = {"csv": po2csv.po2csv, "xlf": po2xliff.po2xliff, "ts": po2ts.po2ts, "mo": pocompile.POCompile}
+    converters = {"csv": po2csv.po2csv, "xlf": po2xliff.po2xliff, "po": xliff2po.xliff2po, "ts": po2ts.po2ts, "mo": pocompile.POCompile}
     converterclass = converters.get(destformat, None)
     if converterclass is None:
       raise ValueError("No converter available for %s" % destfilename)
@@ -1318,15 +1293,15 @@ class TranslationProject(object):
   def gettext(self, message):
     """uses the project as a live translator for the given message"""
     for pofilename, pofile in self.pofiles.iteritems():
-      if pofile.pomtime != pootlefile.getmodtime(pofile.filename):
+      if pofile.pomtime != statistics.getmodtime(pofile.filename):
         pofile.readpofile()
         pofile.makeindex()
       elif not hasattr(pofile, "sourceindex"):
         pofile.makeindex()
-      thepo = pofile.sourceindex.get(message, None)
-      if not thepo or thepo.isblankmsgstr():
+      unit = pofile.sourceindex.get(message, None)
+      if not unit or unit.isblankmsgstr():
         continue
-      tmsg = thepo.target
+      tmsg = unit.target
       if tmsg is not None:
         return tmsg
     return message
@@ -1335,15 +1310,15 @@ class TranslationProject(object):
     """gets the translation of the message by searching through all the pofiles (unicode version)"""
     for pofilename, pofile in self.pofiles.iteritems():
       try:
-        if pofile.pomtime != pootlefile.getmodtime(pofile.filename):
+        if pofile.pomtime != statistics.getmodtime(pofile.filename):
           pofile.readpofile()
           pofile.makeindex()
         elif not hasattr(pofile, "sourceindex"):
           pofile.makeindex()
-        thepo = pofile.sourceindex.get(message, None)
-        if not thepo or thepo.isblankmsgstr() or thepo.isfuzzy():
+        unit = pofile.sourceindex.get(message, None)
+        if not unit or unit.isblankmsgstr() or unit.isfuzzy():
           continue
-        tmsg = thepo.target
+        tmsg = unit.target
         if tmsg is not None:
           if isinstance(tmsg, unicode):
             return tmsg
@@ -1357,7 +1332,7 @@ class TranslationProject(object):
     """gets the plural translation of the message by searching through all the pofiles (unicode version)"""
     for pofilename, pofile in self.pofiles.iteritems():
       try:
-        if pofile.pomtime != pootlefile.getmodtime(pofile.filename):
+        if pofile.pomtime != statistics.getmodtime(pofile.filename):
           pofile.readpofile()
           pofile.makeindex()
         elif not hasattr(pofile, "sourceindex"):
@@ -1365,10 +1340,10 @@ class TranslationProject(object):
         nplural, pluralequation = pofile.getheaderplural()
         if pluralequation:
           pluralfn = gettext.c2py(pluralequation)
-          thepo = pofile.sourceindex.get(singular, None)
-          if not thepo or thepo.isblankmsgstr() or thepo.isfuzzy():
+          unit = pofile.sourceindex.get(singular, None)
+          if not unit or unit.isblankmsgstr() or unit.isfuzzy():
             continue
-          tmsg = po.unquotefrompo(thepo.msgstr[pluralfn(n)])
+          tmsg = unit.target.strings[pluralfn(n)]
           if tmsg is not None:
             if isinstance(tmsg, unicode):
               return tmsg
@@ -1436,6 +1411,8 @@ class TemplatesProject(TranslationProject):
   def getrights(self, session=None, username=None, usedefaults=True):
     """gets the rights for the given user (name or session, or not-logged-in if username is None)"""
     # internal admin sessions have all rights
+    # We don't send the usedefaults parameter through, because we don't want users of this method to
+    # change the default behaviour in a template project. Yes, I know: ignorance and deceit.
     rights = super(TemplatesProject, self).getrights(session=session, username=username)
     if rights is not None:
       rights = [right for right in rights if right not in ["translate", "suggest", "pocompile"]]
