@@ -19,11 +19,12 @@
 # along with translate; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import sre
+import re
 from jToolkit import spellcheck
 from Pootle import pagelayout
 from Pootle import projects
 from Pootle import pootlefile
+from translate.storage import po
 import difflib
 import urllib
 
@@ -55,7 +56,8 @@ class TranslatePage(pagelayout.PootleNavPage):
     self.pofilename = self.argdict.pop("pofilename", None)
     if self.pofilename == "":
       self.pofilename = None
-    if self.pofilename is None and self.dirfilter is not None and self.dirfilter.endswith(".po"):
+    if self.pofilename is None and self.dirfilter is not None and \
+            (self.dirfilter.endswith(".po") or self.dirfilter.endswith(".xlf")):
       self.pofilename = self.dirfilter
     self.receivetranslations()
     # TODO: clean up modes to be one variable
@@ -73,20 +75,20 @@ class TranslatePage(pagelayout.PootleNavPage):
     givenpofilename = self.pofilename
     formaction = self.makelink("")
     mainstats = ""
+    pagelinks = None
+    if self.viewmode:
+      rows = self.getdisplayrows("view")
+      icon="file"
+    else:
+      rows = self.getdisplayrows("translate")
+      icon="edit"
     if self.pofilename is not None:
       postats = self.project.getpostats(self.pofilename)
       blank, fuzzy = postats["blank"], postats["fuzzy"]
       translated, total = postats["translated"], postats["total"]
       mainstats = self.localize("%d/%d translated\n(%d blank, %d fuzzy)", len(translated), len(total), len(blank), len(fuzzy))
-    if self.viewmode:
-      rows = self.getdisplayrows("view")
       pagelinks = self.getpagelinks("?translate=1&view=1", rows)
-      icon="file"
-    else:
-      rows = self.getdisplayrows("translate")
-      pagelinks = self.getpagelinks("?translate=1&view=1", rows)
-      icon="edit"
-    navbarpath_dict = self.makenavbarpath_dict(self.project, self.session, self.pofilename)
+    navbarpath_dict = self.makenavbarpath_dict(self.project, self.session, self.pofilename, dirfilter=self.dirfilter or "")
     # templatising
     templatename = "translatepage"
     instancetitle = getattr(session.instance, "title", session.localize("Pootle Demo"))
@@ -108,12 +110,19 @@ class TranslatePage(pagelayout.PootleNavPage):
         # translation form
         "actionurl": formaction,
         "notice": notice,
+        # l10n: Heading above the table column with the source language
         "original_title": self.localize("Original"),
+        # l10n: Heading above the table column with the target language
         "translation_title": self.localize("Translation"),
         "items": items,
         "reviewmode": self.reviewmode,
         "accept_button": self.localize("Accept"),
         "reject_button": self.localize("Reject"),
+        "fuzzytext": self.localize("Fuzzy"),
+        # l10n: Heading above the textarea for translator comments.
+        "translator_comments_title": self.localize("Translator comments"),
+        # l10n: Heading above the comments extracted from the programing source code
+        "developer_comments_title": self.localize("Developer comments"),
         # l10n: This heading refers to related translations and terminology
         "related_title": self.localize("Related"),
         # optional sections, will appear if these values are replaced
@@ -128,7 +137,7 @@ class TranslatePage(pagelayout.PootleNavPage):
     if self.showassigns and "assign" in self.rights:
       templatevars["assign"] = self.getassignbox()
     pagelayout.PootleNavPage.__init__(self, templatename, templatevars, session, bannerheight=81)
-    self.addfilelinks(self.pofilename, self.matchnames)
+    self.addfilelinks()
 
   def getfinishedtext(self, stoppedby):
     """gets notice to display when the translation is finished"""
@@ -141,7 +150,7 @@ class TranslatePage(pagelayout.PootleNavPage):
 
   def getpagelinks(self, baselink, pagesize):
     """gets links to other pages of items, based on the given baselink"""
-    baselink += "pofilename=%s" % self.pofilename
+    baselink += "&pofilename=%s" % self.pofilename
     pagelinks = []
     pofilelen = self.project.getpofilelen(self.pofilename)
     if pofilelen <= pagesize or self.firstitem is None:
@@ -183,13 +192,13 @@ class TranslatePage(pagelayout.PootleNavPage):
         pagelink["sep"] = ""
     return pagelinks
 
-  def addfilelinks(self, pofilename, matchnames):
+  def addfilelinks(self):
     """adds a section on the current file, including any checks happening"""
     if self.showassigns and "assign" in self.rights:
       self.templatevars["assigns"] = self.getassignbox()
     if self.pofilename is not None:
-      if matchnames:
-        checknames = [matchname.replace("check-", "", 1) for matchname in matchnames]
+      if self.matchnames:
+        checknames = [matchname.replace("check-", "", 1) for matchname in self.matchnames]
         # TODO: put the following parameter in quotes, since it will be foreign in all target languages
         # l10n: the parameter is the name of one of the quality checks, like "fuzzy"
         self.templatevars["checking_text"] = self.localize("checking %s", ", ".join(checknames))
@@ -218,8 +227,9 @@ class TranslatePage(pagelayout.PootleNavPage):
     rejects = []
     translations = {}
     suggestions = {}
-    pluralitems = {}
-    keymatcher = sre.compile("(\D+)([0-9.]+)")
+    comments = {}
+    fuzzies = {}
+    keymatcher = re.compile("(\D+)([0-9.]+)")
     def parsekey(key):
       match = keymatcher.match(key)
       if match:
@@ -254,6 +264,12 @@ class TranslatePage(pagelayout.PootleNavPage):
         accepts.append((item, pointitem))
       elif keytype == "reject":
         rejects.append((item, pointitem))
+      elif keytype == "translator_comments":
+        # We need to remove carriage returns from the input.
+        value = value.replace("\r", "")
+        comments[item] = value
+      elif keytype == "fuzzy":
+        fuzzies[item] = value
       elif keytype == "trans":
         value = self.unescapesubmition(value)
         if pointitem is not None:
@@ -268,6 +284,8 @@ class TranslatePage(pagelayout.PootleNavPage):
       else:
         continue
       delkeys.append(key)
+
+
     for key in delkeys:
       del self.argdict[key]
     for item in skips:
@@ -280,13 +298,26 @@ class TranslatePage(pagelayout.PootleNavPage):
       value = translations[item]
       self.project.suggesttranslation(self.pofilename, item, value, self.session)
       self.lastitem = item
+      
     for item in submits:
       if item in skips or item not in translations:
         continue
-      value = translations[item]
-      if isinstance(value, dict) and len(value) == 1 and 0 in value:
-        value = value[0]
-      self.project.updatetranslation(self.pofilename, item, value, self.session)
+
+      newvalues = {}
+      newvalues["target"] = translations[item]
+      if isinstance(newvalues["target"], dict) and len(newvalues["target"]) == 1 and 0 in newvalues["target"]:
+        newvalues["target"] = newvalues["target"][0]
+
+      newvalues["fuzzy"] = False
+      if (fuzzies.get(item) == u'on'):
+        newvalues["fuzzy"] = True
+
+      translator_comments = comments.get(item)
+      if translator_comments:
+        newvalues["translator_comments"] = translator_comments
+
+      self.project.updatetranslation(self.pofilename, item, newvalues, self.session)
+      
       self.lastitem = item
     for item, suggid in rejects:
       value = suggestions[item, suggid]
@@ -317,7 +348,7 @@ class TranslatePage(pagelayout.PootleNavPage):
   def getusernode(self):
     """gets the user's prefs node"""
     if self.session.isopen:
-      return getattr(self.session.loginchecker.users, self.session.username, None)
+      return getattr(self.session.loginchecker.users, self.session.username.encode("utf-8"), None)
     else:
       return None
 
@@ -392,10 +423,10 @@ class TranslatePage(pagelayout.PootleNavPage):
     items = []
     if self.reviewmode and self.item is not None:
       suggestions = {self.item: self.project.getsuggestions(self.pofilename, self.item)}
-    for row, thepo in enumerate(self.translations):
+    for row, unit in enumerate(self.translations):
       tmsuggestions = []
-      orig = thepo.unquotedmsgid
-      trans = thepo.unquotedmsgstr
+      orig = unit.source.strings
+      trans = unit.target.strings
       nplurals, plurals = self.project.getpofile(self.pofilename).getheaderplural()
       try:
         if len(orig) > 1:
@@ -414,19 +445,26 @@ class TranslatePage(pagelayout.PootleNavPage):
       item = self.firstitem + row
       origdict = self.getorigdict(item, orig, item in self.editable)
       transmerge = {}
+
+      message_context = ""
       if item in self.editable:
-        comments = self.escapetext(thepo.getnotes(), stripescapes=True)
-        locations = " ".join(thepo.getlocations())
+        translator_comments = unit.getnotes(origin="translator")
+        developer_comments = self.escapetext(unit.getnotes(origin="developer"), stripescapes=True)
+        locations = " ".join(unit.getlocations())
+        if isinstance(unit, po.pounit):
+          message_context = "".join(unit.getcontext())
         tmsuggestions = self.project.gettmsuggestions(self.pofilename, self.item)
         tmsuggestions.extend(self.project.getterminology(self.session, self.pofilename, self.item))
         
         if self.reviewmode:
-          itemsuggestions = [suggestion.unquotedmsgstr for suggestion in suggestions[item]]
+          translator_comments = self.escapetext(unit.getnotes(origin="translator"), stripescapes=True)
+          itemsuggestions = [suggestion.target.strings for suggestion in suggestions[item]]
           transmerge = self.gettransreview(item, trans, itemsuggestions)
         else:
           transmerge = self.gettransedit(item, trans)
       else:
-        comments = thepo.getnotes()
+        translator_comments = unit.getnotes(origin="translator")
+        developer_comments = unit.getnotes(origin="developer")
         locations = ""
         transmerge = self.gettransview(item, trans)
       transdict = {"itemid": "trans%d" % item,
@@ -435,16 +473,16 @@ class TranslatePage(pagelayout.PootleNavPage):
                   }
       transdict.update(transmerge)
       polarity = oddoreven(item)
-      origcell_class = "translate-original translate-original-%s" % polarity
-      transcell_class = "translate-translation translate-translation-%s" % polarity
       if item in self.editable:
         focus_class = "translate-focus"
       else:
         focus_class = ""
       
       state_class = ""
-      if thepo.isfuzzy():
+      fuzzy = None
+      if unit.isfuzzy():
         state_class += "translate-translation-fuzzy"
+        fuzzy = "checked"
 
       itemdict = {
                  "itemid": item,
@@ -454,8 +492,11 @@ class TranslatePage(pagelayout.PootleNavPage):
                  "focus_class": focus_class,
                  "editable": item in self.editable,
                  "state_class": state_class,
-                 "comments": comments,
+                 "fuzzy": fuzzy,
+                 "translator_comments": translator_comments,
+                 "developer_comments": developer_comments,
                  "locations": locations,
+                 "message_context": message_context,
                  "tm": tmsuggestions,
                  }
       items.append(itemdict)
@@ -493,13 +534,13 @@ class TranslatePage(pagelayout.PootleNavPage):
   def addfancyspaces(self, text):
     """Insert fancy spaces"""
     #More than two consecutive:
-    text = sre.sub("[ ]{2,}", self.fancyspaces, text)
+    text = re.sub("[ ]{2,}", self.fancyspaces, text)
     #At start of string
-    text = sre.sub("^[ ]+", self.fancyspaces, text)
+    text = re.sub("^[ ]+", self.fancyspaces, text)
     #After newline
-    text = sre.sub("\\n([ ]+)", self.fancyspaces, text)
+    text = re.sub("\\n([ ]+)", self.fancyspaces, text)
     #At end of string
-    text = sre.sub("[ ]+$", self.fancyspaces, text)
+    text = re.sub("[ ]+$", self.fancyspaces, text)
     return text
 
   def escapefortextarea(self, text):
@@ -577,11 +618,6 @@ class TranslatePage(pagelayout.PootleNavPage):
             # l10n: action that decreases the height of the textarea
             "shrink": self.localize("Shrink"),
             # l10n: action that increases the width of the textarea
-            "broaden": self.localize("Broaden"),
-            # l10n: action that decreases the width of the textarea
-            "narrow": self.localize("Narrow"),
-            # l10n: action that resets the size of the textarea
-            "reset": self.localize("Reset")
            }
 
   def gettransedit(self, item, trans):
