@@ -19,18 +19,21 @@
 # along with translate; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from jToolkit import web
 from jToolkit.web import server
-from jToolkit.web import session
 from jToolkit import mailer
 from jToolkit import prefs
 from Pootle import pagelayout
+from translate.lang import data as langdata
+from email.Header import Header
+import locale
 
 class RegistrationError(ValueError):
   def __init__(self, message):
     message = message.encode('utf-8')
     ValueError.__init__(self, message)
 
-# This mimimum passwordlength is mandated by the interface when registering or
+# This mimimum passwordlength is mandated by the interface when registering or 
 # changing password
 minpasswordlen = 6
 
@@ -75,7 +78,11 @@ class LoginPage(pagelayout.PootlePage):
       languageoptions += self.languagenames.items()
     else:
       languageoptions += self.languagenames
-    return [{"code": key, "name": value, "selected": key==session.language or None} for key, value in languageoptions]
+    if session.language in ["en", session.server.defaultlanguage]:
+        preferredlanguage = ""
+    else:
+        preferredlanguage = session.language
+    return [{"code": key, "name": value, "selected": key==preferredlanguage or None} for key, value in languageoptions if key != 'templates']
 
 class RegisterPage(pagelayout.PootlePage):
   """page for new registrations"""
@@ -95,7 +102,7 @@ class RegisterPage(pagelayout.PootlePage):
         "username_tooltip": self.localize("Your requested username"),
         "username": self.argdict.get("username", ""),
         "email_title": self.localize("Email Address"),
-        "email_tooltip": self.localize("Must be a valid email address"),
+        "email_tooltip": self.localize("You must supply a valid email address"),
         "email": self.argdict.get("email", ""),
         "fullname_title": self.localize("Full Name"),
         "fullname_tooltip": self.localize("Your full name"),
@@ -143,6 +150,7 @@ class UserOptions(pagelayout.PootlePage):
     self.potree = potree
     self.session = session
     self.localize = session.localize
+    self.tr_lang = session.tr_lang
     message = forcemessage(message)
     pagetitle = self.localize("Options for: %s", session.username)
     templatename = "options"
@@ -186,7 +194,8 @@ class UserOptions(pagelayout.PootlePage):
     languageoptions = self.potree.getlanguages()
     languages = []
     for language, name in languageoptions:
-      languages.append({"code": language, "name": name, "selected": language in userlanguages or None})
+      languages.append({"code": language, "name": self.tr_lang(name), "selected": language in userlanguages or None})
+    languages.sort(cmp=locale.strcoll, key=lambda dict: dict["name"])
     return languages
 
   def getotheroptions(self):
@@ -197,8 +206,11 @@ class UserOptions(pagelayout.PootlePage):
         uilanguage = userlanguages[0]
     languageoptions = [{"code": '', "name": ''}]
     for code, name in self.potree.getlanguages():
-      languageoptions.append({"code": code, "name": name, "selected": uilanguage == code or None})
-    options = {"inputheight": self.localize("Input Height (in lines)"), "inputwidth": self.localize("Input Width (in characters)"),
+      if code == "templates":
+        continue
+      languageoptions.append({"code": code, "name": self.tr_lang(name), "selected": uilanguage == code or None})
+    languageoptions.sort(cmp=locale.strcoll, key=lambda dict: dict["name"])
+    options = {"inputheight": self.localize("Input Height (in lines)"), 
           "viewrows": self.localize("Number of rows in view mode"), 
           "translaterows": self.localize("Number of rows in translate mode")}
     optionlist = []
@@ -211,6 +223,7 @@ class OptionalLoginAppServer(server.LoginAppServer):
   """a server that enables login but doesn't require it except for specified pages"""
   def handle(self, req, pathwords, argdict):
     """handles the request and returns a page object in response"""
+    session = None
     try:
       argdict = self.processargs(argdict)
       session = self.getsession(req, argdict)
@@ -221,18 +234,28 @@ class OptionalLoginAppServer(server.LoginAppServer):
         self.initlanguage(req, session)
       page = self.getpage(pathwords, session, argdict)
     except Exception, e:
+      # Because of the exception, 'session' might not be initialised. So let's
+      # play extra safe
+      if not session:
+          raise Exception("Could not initialise session.\nDetail:%s" % str(e))
+
       exceptionstr = self.errorhandler.exception_str()
       errormessage = str(e).decode("utf-8")
       traceback = self.errorhandler.traceback_str().decode('utf-8')
       browsertraceback = ""
-      if self.options.browsererrors == 'traceback':
-        browsertraceback = traceback
-      if self.options.logerrors == 'traceback':
+      options = getattr(self, "options", None)
+      # with unit tests we might not have self.options, therefore this test
+      if options:
+        if options.browsererrors == 'traceback':
+          browsertraceback = traceback
+        if options.logerrors == 'traceback':
+          self.errorhandler.logerror(traceback)
+        elif options.logerrors == 'exception':
+          self.errorhandler.logerror(exceptionstr)
+        elif options.logerrors == 'message':
+          self.errorhandler.logerror(errormessage)
+      else:
         self.errorhandler.logerror(traceback)
-      elif self.options.logerrors == 'exception':
-        self.errorhandler.logerror(exceptionstr)
-      elif self.options.logerrors == 'message':
-        self.errorhandler.logerror(errormessage)
       
       refreshurl = req.headers_in.getheader('Referer') or "/"
       templatename = "error"
@@ -241,8 +264,8 @@ class OptionalLoginAppServer(server.LoginAppServer):
           "refresh": 30,
           "refreshurl": refreshurl,
           "message": errormessage,
-	  "traceback": browsertraceback,
-	  "back": session.localize("Back"),
+          "traceback": browsertraceback,
+          "back": session.localize("Back"),
           }
       pagelayout.completetemplatevars(templatevars, session)
       page = server.Redirect(refreshurl, withtemplate=(templatename, templatevars))
@@ -272,7 +295,7 @@ class OptionalLoginAppServer(server.LoginAppServer):
     usernode = self.getusernode(users, username)
     usernode.name = fullname
     usernode.email = email
-    usernode.passwdhash = session.md5hexdigest(password)
+    usernode.passwdhash = web.session.md5hexdigest(password)
 
   def makeactivationcode(self, users, username):
     """makes a new activation code for the user and returns it"""
@@ -315,7 +338,7 @@ class OptionalLoginAppServer(server.LoginAppServer):
         if self.hasuser(users, username):
           usernode = self.getusernode(users, username)
           if value and value.strip():
-            usernode.passwdhash = session.md5hexdigest(value.strip())
+            usernode.passwdhash = web.session.md5hexdigest(value.strip())
       elif key.startswith("useractivated-"):
         username = key.replace("useractivated-", "", 1)
         self.activate(users, username)
@@ -405,7 +428,9 @@ class OptionalLoginAppServer(server.LoginAppServer):
     message += session.localize("Your registered email address is: %s\n", email)
     smtpserver = self.instance.registration.smtpserver
     fromaddress = self.instance.registration.fromaddress
-    messagedict = {"from": fromaddress, "to": [email], "subject": session.localize("Pootle Registration"), "body": message}
+    subject = Header(session.localize("Pootle Registration"),
+                     "utf-8").encode()
+    messagedict = {"from": fromaddress, "to": [email], "subject": subject, "body": message}
     if supportaddress:
       messagedict["reply-to"] = supportaddress
     fullmessage = mailer.makemessage(messagedict)
@@ -426,6 +451,8 @@ class OptionalLoginAppServer(server.LoginAppServer):
       redirectpage = pagelayout.PootlePage("Redirecting...", {}, session)
       redirectpage.templatename = "redirect"
       redirectpage.templatevars = {
+          # BUG: We won't redirect to registration page, we will go to 
+          # activation or login
           "pagetitle": session.localize("Redirecting to Registration Page..."),
           "refresh": 10,
           "refreshurl": redirecturl,
@@ -462,7 +489,7 @@ class OptionalLoginAppServer(server.LoginAppServer):
     else:
       return ActivatePage(session, argdict)
 
-class PootleSession(session.LoginSession):
+class PootleSession(web.session.LoginSession):
   """a session object that knows about Pootle"""
   def __init__(self, sessioncache, server, sessionstring = None, loginchecker = None):
     """sets up the session and remembers the users prefs"""
@@ -513,6 +540,13 @@ class PootleSession(session.LoginSession):
       if not getattr(self.prefs, "uilanguage", "") and self.language_set:
         self.setinterfaceoptions({"option-uilanguage": self.language_set})
     self.translation = self.server.gettranslation(self.language)
+    self.tr_lang = langdata.tr_lang(self.language)
+    try:
+        locale.setlocale(locale.LC_ALL, str(self.language))
+    except locale.Error:
+        # The system might not have the locale installed
+        pass
+    self.checkstatus(None, None)
 
   def validate(self):
     """checks if this session is valid (which means the user must be activated)"""
@@ -550,7 +584,7 @@ class PootleSession(session.LoginSession):
     setattr(self.prefs, "name", name)
     setattr(self.prefs, "email", email)
     if password:
-      passwdhash = session.md5hexdigest(argdict.get("option-password", ""))
+      passwdhash = web.session.md5hexdigest(argdict.get("option-password", ""))
       setattr(self.prefs, "passwdhash", passwdhash)
     self.saveprefs()
 
